// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package announcement

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkAnnouncementDelParam(req *pb_v1.AnnouncementDelRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	}
	return nil

}

func AnnouncementDel(ctx context.Context, req *pb_v1.AnnouncementDelRequest) (reply *pb_v1.AnnouncementDelReply, err error) {

	reply = &pb_v1.AnnouncementDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkAnnouncementDelParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	// 获取旧数据
	old := dbmodel.NewAnnouncement(dbname)
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.AnnouncementUpdateRequest{
		GardenId: req.GardenId,
		Id:       req.Id,
		Content:  old.Content,
		Title:    old.Title,
	}

	// 删除数据
	p := dbmodel.NewAnnouncement(dbname)

	db := database.DB().Begin()
	err = p.Delete(db, where)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	ar := dbmodel.NewAnnouncementRead(dbname)
	where = map[string]interface{}{
		"announcement_id": req.Id,
	}
	err = ar.Delete(db, where)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	outList := oss_utils.GetObjFromText(old.Content)
	if len(outList) > 0 {
		if err := oss_utils.OssObjAdd(nil, outList); err != nil {
			db.Rollback()
			return nil, err
		}
	}

	db.Commit()

	return reply, nil
}
