// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package announcement

import (
	"context"
	"encoding/json"
	"fmt"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkAnnouncementListParam(req *pb_v1.AnnouncementListRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}

	return nil

}

func readInfo(ids []int64, uid int64, dbname string) (map[int64]bool, error) {

	ret := map[int64]bool{}

	if uid == 0 || len(ids) == 0 {
		return ret, nil
	}
	p := dbmodel.NewAnnouncementRead(dbname)
	where := map[string]interface{}{
		"announcement_id in": ids,
		"uid":                uid,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range list {
		ret[v.AnnouncementId] = true
	}
	return ret, nil

}

func AnnouncementList(ctx context.Context, req *pb_v1.AnnouncementListRequest) (reply *pb_v1.AnnouncementListReply, err error) {

	reply = &pb_v1.AnnouncementListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkAnnouncementListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	p := dbmodel.NewAnnouncement(dbname)
	where := map[string]interface{}{
		//"garden_id":req.GardenId,
	}
	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.AnnouncementItem, len(list))

	ids := make([]int64, len(list))
	for i, v := range list {
		ids[i] = v.ID
	}
	info, err := readInfo(ids, req.Uid, dbname)
	if err != nil {
		return nil, err
	}
	fmt.Println(info)
	for i, v := range list {

		item := &pb_v1.AnnouncementItem{
			Title:     v.Title,
			Content:   v.Content,
			Id:        v.ID,
			ReadCount: v.ReadCount,
			Read:      info[v.ID],
			CreatedAt: v.CreatedAt.Format("2006-01-02 15:04:05"),
		}

		reply.List[i] = item
	}

	return reply, nil
}
