// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package announcement

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkAnnouncementReadAddParam(req *pb_v1.AnnouncementReadAddRequest) error {

	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Uid == 0:
		return status.Error(10003, "uid不能为空")
	case req.Id == 0:
		return status.Error(10003, "公告不能为空")
	}
	return nil
}

func AnnouncementReadAdd(ctx context.Context, req *pb_v1.AnnouncementReadAddRequest) (reply *pb_v1.AnnouncementReadAddReply, err error) {

	reply = &pb_v1.AnnouncementReadAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkAnnouncementReadAddParam(req)
	if err != nil {
		return nil, err
	}
	now := time.Now()
	dbname := utils.GetGardenDbName(req.GardenId)

	p := dbmodel.NewAnnouncementRead(dbname)
	p.AnnouncementId = req.Id
	p.Uid = req.Uid
	p.CreatedAt = now
	p.UpdatedAt = now
	db := database.DB().Begin()
	err = p.Insert(db)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			db.Rollback()
			return reply, nil
		}
	}

	a := dbmodel.NewAnnouncement(dbname)
	where := map[string]interface{}{
		"id": req.Id,
	}
	values := map[string]interface{}{
		"read_count": gorm.Expr("read_count + ?", 1),
	}
	err = a.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	db.Commit()

	return reply, nil
}
