// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package announcement

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkAnnouncementUpdateParam(req *pb_v1.AnnouncementUpdateRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Title == "":
		return status.Error(10003, "标题不能为空")
	case req.Content == "":
		return status.Error(10003, "内容不能为空")
	}
	return nil
}




func AnnouncementUpdate(ctx context.Context, req *pb_v1.AnnouncementUpdateRequest) (reply *pb_v1.AnnouncementUpdateReply, err error) {

	reply = &pb_v1.AnnouncementUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkAnnouncementUpdateParam(req)
	if err != nil {
		return nil, err
	}


	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewAnnouncement(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.AnnouncementUpdateRequest{
		GardenId:req.GardenId,
		Id:req.Id,
		Content:req.Content,
		Title:req.Title,
	}



	// 更新新数据
	now := time.Now()
	p := dbmodel.NewAnnouncement(dbname)

	values := map[string]interface{}{
		"title":req.Title,
		"content":req.Content,
		"updated_at":now,

	}

	where = map[string]interface{}{
		"id":req.Id,
	}
	db := database.DB().Begin()
	err = p.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	inList := oss_utils.GetObjFromText(req.Content)
	outList := oss_utils.GetObjFromText(old.Content)
	if len(outList) > 0  || len(outList) > 0{
		if err := oss_utils.OssObjAdd(inList, outList); err != nil {
			db.Rollback()
			return nil, err
		}
	}


	db.Commit()

	return reply, nil
}
