// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkChargeHouseBindedListParam(req *pb_v1.ChargeHouseBindedListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.ChargeId < 1:
		return status.Error(10003, "费用项目不能为空")
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	return nil

}


func ChargeHouseBindedList(ctx context.Context, req *pb_v1.ChargeHouseBindedListRequest) (reply *pb_v1.ChargeHouseBindedListReply, err error) {

	reply = &pb_v1.ChargeHouseBindedListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeHouseBindedListParam(req)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	dbname := utils.GetGardenDbName(req.GardenId)


	exist := fmt.Sprintf("exists(select obj_id from %s.t_charge_bind where t_charge_bind.obj_id=t_house_unit_building.id and t_charge_bind.charge_id=? and t_charge_bind.obj_type = %d)", dbname, charge_utils.ObjTypeHouse)
	house := dbmodel.NewHouseUnitBuilding(dbname)
	where := map[string]interface{}{
		exist:req.ChargeId,
	}
	reply.Total, err = house.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := house.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}

	reply.List = make([]*pb_v1.ChargeHouseData, len(list))
	ids := make([]int64, len(list))
	for i, v := range list {
		ids[i] = v.ID
	}

	bind := dbmodel.NewChargeBind(dbname)
	where = map[string]interface{}{
		"obj_id in":ids,
		"obj_type":charge_utils.ObjTypeHouse,
		"charge_id":req.ChargeId,
	}

	blist, _ := bind.List(database.DB(), where, nil, -1, -1)
	m := map[int64]dbmodel.TChargeBind{}
	for _, v := range blist {
		m[v.ObjId] = v
	}


	reply.List = make([]*pb_v1.ChargeHouseData, len(list))
	for i, v := range list {
		item := &pb_v1.ChargeHouseData{
			HouseId:v.ID,
			HouseArea:v.HouseArea,
			HouseUsedArea:v.HouseUsedArea,
			HouseNumber:v.HouseNumber,
			Layer:v.Layer,
			HouseType:int32(v.HouseType),
			HouseStatus:int32(v.Status),
			Start:m[v.ID].Start,
			HouseName:fmt.Sprintf("%v-%v-%v",
				v.BuildingNumber,
				v.UnitNumber,
				v.HouseNumber),

		}
		reply.List[i] = item
	}

	return reply, nil
}
