// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkChargeHouseGroupParam(req *pb_v1.ChargeHouseGroupRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	return nil

}

func ChargeHouseGroup(ctx context.Context, req *pb_v1.ChargeHouseGroupRequest) (reply *pb_v1.ChargeHouseGroupReply, err error) {

	reply = &pb_v1.ChargeHouseGroupReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeHouseGroupParam(req)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	dbname := utils.GetGardenDbName(req.GardenId)

	exist := fmt.Sprintf("exists(select obj_id from %s.t_charge_bind where t_charge_bind.obj_id=t_house_unit_building.id  and t_charge_bind.obj_type = %d)", dbname, charge_utils.ObjTypeHouse)
	house := dbmodel.NewHouseUnitBuilding(dbname)
	where := map[string]interface{}{
		exist: "",
	}
	if req.ObjId > 0 {
		where["id"] = req.ObjId
	}
	reply.Total, err = house.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := house.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	houseIds := make([]int64, len(list))
	for i, v := range list {
		houseIds[i] = v.ID
	}
	bind := dbmodel.NewChargeBind(dbname)
	where = map[string]interface{}{
		"obj_id in":   houseIds,
		"obj_type":    charge_utils.ObjTypeHouse,
		"charge_type": charge_utils.ChargeTypeProperty,
	}

	blist, err := bind.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	m := map[int64]int64{}

	for _, v := range blist {
		m[v.ObjId] = v.ID
	}

	bill := dbmodel.NewChargeBill(dbname)
	where = map[string]interface{}{
		"obj_id in": houseIds,
		"obj_type":  charge_utils.ObjTypeHouse,
	}
	type Bcount struct {
		Count int64 `json:"count"`
		ObjId int64 `json:"obj_id"`
	}
	billlist := []Bcount{}
	err = bill.SelectGroup(database.DB(), where, nil, "sum(1) as count, obj_id", "obj_id", -1, -1, &billlist)
	if err != nil {
		return nil, errors.DataBaseError
	}
	bm := map[int64]bool{}
	for _, v := range billlist {
		if v.Count > 0 {
			bm[v.ObjId] = true
		}
	}

	reply.List = make([]*pb_v1.ChargeHouseGroupItem, len(list))
	for i, v := range list {
		item := &pb_v1.ChargeHouseGroupItem{
			HouseId:       v.ID,
			HouseArea:     v.HouseArea,
			HouseUsedArea: v.HouseUsedArea,
			Layer:         v.Layer,
			HouseType:     int32(v.HouseType),
			HouseStatus:   int32(v.Status),
			HouseName: fmt.Sprintf("%v-%v-%v",
				v.BuildingNumber,
				v.UnitNumber,
				v.HouseNumber),
			HousePropertyBindId: m[v.ID],
			HasUnpayBill:        bm[v.ID],
		}
		reply.List[i] = item
	}

	return reply, nil
}
