// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"sort"
)

func checkChargeMonthInfoParam(req *pb_v1.ChargeMonthInfoRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.ObjId == 0 || req.ObjType == 0:
		return status.Error(10003, "对象id和type不能为空")
	case req.ChargeType == 0:
		return status.Error(10003, "费用类型不能为空")
	}

	return nil

}

func makePropertyDesc(bind *dbmodel.TChargeBind, chargeConf *dbmodel.TChargeConf) string {
	desc := ""
	switch {
	case chargeConf.UnitPrice > 0 && chargeConf.FixAmount == 0:
		desc = fmt.Sprintf("房屋面积：%.2f;单价:%.2f",
			bind.ObjArea, float64(chargeConf.UnitPrice)/100.00)
	case chargeConf.UnitPrice > 0 && chargeConf.FixAmount > 0:
		fixName := chargeConf.FixAmountName
		if fixName == "" {
			fixName = "附加费"
		}
		desc = fmt.Sprintf("房屋面积：%.2f;单价:%.2f;%s:%.2f",
			bind.ObjArea, float64(chargeConf.UnitPrice)/100.00, fixName, float64(chargeConf.FixAmount)/100.00)

	case chargeConf.UnitPrice == 0 && chargeConf.FixAmount > 0:
		fixName := chargeConf.FixAmountName
		if fixName == "" {
			fixName = "固定费用"
		}
		desc = fmt.Sprintf("房屋面积：%.2f;%s:%.2f",
			bind.ObjArea, fixName, float64(chargeConf.FixAmount)/100.00)

	}
	return desc
}

func makeSpaceDesc(bind *dbmodel.TChargeBind, chargeConf *dbmodel.TChargeConf) string {
	desc := ""
	switch {
	case chargeConf.UnitPrice > 0 && chargeConf.FixAmount == 0:
		desc = fmt.Sprintf("车位面积：%.2f;单价:%.2f",
			bind.ObjArea, float64(chargeConf.UnitPrice)/100.00)
	case chargeConf.UnitPrice > 0 && chargeConf.FixAmount > 0:
		fixName := chargeConf.FixAmountName
		if fixName == "" {
			fixName = "附加费"
		}
		desc = fmt.Sprintf("车位面积：%.2f;单价:%.2f;%s:%.2f",
			bind.ObjArea, float64(chargeConf.UnitPrice)/100.00, fixName, float64(chargeConf.FixAmount)/100.00)

	case chargeConf.UnitPrice == 0 && chargeConf.FixAmount > 0:
		fixName := chargeConf.FixAmountName
		if fixName == "" {
			fixName = "固定费用"
		}
		desc = fmt.Sprintf("车位面积：%.2f;%s:%.2f",
			bind.ObjArea, fixName, float64(chargeConf.FixAmount)/100.00)

	}
	return desc
}

func makeVehicleDesc(bind *dbmodel.TChargeBind, chargeConf *dbmodel.TChargeConf) string {
	desc := ""
	switch {
	case bind.CustomFee > 0:
		desc = fmt.Sprintf("月租停车费:%.2f",
			float64(bind.CustomFee)/100.00)
	case chargeConf.FixAmount > 0:
		desc = fmt.Sprintf("月租停车费:%.2f",
			float64(chargeConf.FixAmount)/100.00)
	}
	return desc
}

func makeDesc(bind *dbmodel.TChargeBind, chargeConf *dbmodel.TChargeConf) string {
	switch bind.ObjType {
	case charge_utils.ObjTypeHouse:
		return makePropertyDesc(bind, chargeConf)
	case charge_utils.ObjTypeSpace:
		return makeSpaceDesc(bind, chargeConf)
	case charge_utils.ObjTypeVehicle:
		return makeVehicleDesc(bind, chargeConf)
	}
	return ""
}

func ChargeMonthInfo(ctx context.Context, req *pb_v1.ChargeMonthInfoRequest) (reply *pb_v1.ChargeMonthInfoReply, err error) {

	reply = &pb_v1.ChargeMonthInfoReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeMonthInfoParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)

	bind := dbmodel.NewChargeBind(dbname)

	where := map[string]interface{}{
		"obj_id":      req.ObjId,
		"obj_type":    req.ObjType,
		"charge_type": req.ChargeType,
	}
	err = bind.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if bind.ID == 0 {
		return nil, status.Error(10003, "暂未绑定费用")
	}

	chargeConf := dbmodel.NewChargeConf(dbname)
	where = map[string]interface{}{
		"id": bind.ChargeId,
	}
	err = chargeConf.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if chargeConf.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	var defaultMonths = map[int64]pb_v1.MonthInfo{
		1:  pb_v1.MonthInfo{Months: 1},
		2:  pb_v1.MonthInfo{Months: 2},
		3:  pb_v1.MonthInfo{Months: 3},
		6:  pb_v1.MonthInfo{Months: 6},
		12: pb_v1.MonthInfo{Months: 12},
	}
	if req.ChargeType == charge_utils.ChargeTypeProperty {
		propertyPackage := dbmodel.NewPropertyPackage(dbname)
		filter := map[string]interface{}{
			"enable": 1,
		}
		list, err := propertyPackage.List(database.DB(), filter, nil, -1, -1)
		if err != nil {
			return nil, errors.DataBaseError
		}
		for _, v := range list {
			if v.Enable == 2 {
				continue
			}
			if defaultMonths[v.PayMonths].GiveMonths > 0 {
				continue
			}
			defaultMonths[v.PayMonths] = pb_v1.MonthInfo{
				PackageId:  v.ID,
				Months:     v.PayMonths,
				GiveMonths: v.GiveMonths,
				GiveText:   v.GiveContent,
			}
		}
	}

	array := make([]int, len(defaultMonths))
	i := 0
	for month, _ := range defaultMonths {
		array[i] = int(month)
		i++
	}
	sort.Ints(array)
	packageList := make([]*pb_v1.MonthInfo, len(array))
	for i, month := range array {
		tmp := defaultMonths[int64(month)]
		packageList[i] = &tmp
	}
	// 比如 3 赠 1，则大于3 的也应赠1
	last := -1
	for i, v := range packageList {
		if v.PackageId > 0 {
			last = i
			continue
		}
		if last > -1 {
			packageList[i].PackageId = packageList[last].PackageId
			packageList[i].GiveMonths = packageList[last].GiveMonths
			packageList[i].GiveText = packageList[last].GiveText
		}
	}

	reply.BindId = bind.ID
	reply.Desc = makeDesc(bind, chargeConf)
	reply.MonthList = packageList

	return reply, nil
}
