// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"
)



func checkChargeOrderListParam(req *pb_v1.ChargeOrderListRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	return nil

}


func ChargeOrderList(ctx context.Context, req *pb_v1.ChargeOrderListRequest) (reply *pb_v1.ChargeOrderListReply, err error) {

	reply = &pb_v1.ChargeOrderListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeOrderListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)

	order := dbmodel.NewChargeOrder(dbname)
	where := map[string]interface{}{}
	where["deleted"] = 0
	if req.PayType > 0 {
		where["pay_type"] = req.PayType
	}
	if req.ObjName != "" {
		where["obj_name"] = req.ObjName
	}
	if req.PayStatus > 0 {
		where["status"] = req.PayStatus
	}
	if req.Uid > 0 {
		where["uid"] = req.Uid
	}
	// 账单数量
	reply.Total, err = order.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}

	reply.Page = req.Page

	// 账单列表
	list, err := order.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.ChargeOrderItem, len(list))
	for i, v := range list {

		item := &pb_v1.ChargeOrderItem{
			TransactionId:v.TransactionId,
			PayingDesc:v.PayingDesc,
			ShouldPayAmount:v.Amount,
			PayAmount:v.PayAmount,
			Comment:v.Comment,
			PayTime:time.Unix(v.PayTime, 0).Format("2006-01-02 15:04:05"),
			PayType:v.PayType,
			ObjName:v.ObjName,
			OrderId:v.OrderId,
			Status:int32(v.Status),
			Desc:v.PayingDesc,
		}
		billCount := len(strings.Split(v.BillIds, ","))
		if billCount == 0 && v.OnlinePay == 1 {
			billCount = 1
		}
		item.BillCount = int64(billCount)


		reply.List[i] = item
	}


	return reply, nil
}
