// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkChargeSpaceGroupParam(req *pb_v1.ChargeSpaceGroupRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")

	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	return nil

}

func ChargeSpaceGroup(ctx context.Context, req *pb_v1.ChargeSpaceGroupRequest) (reply *pb_v1.ChargeSpaceGroupReply, err error) {

	reply = &pb_v1.ChargeSpaceGroupReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeSpaceGroupParam(req)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	dbname := utils.GetGardenDbName(req.GardenId)

	exist := fmt.Sprintf("exists(select obj_id from %s.t_charge_bind where t_charge_bind.obj_id=t_space_park.id and t_charge_bind.obj_type = %d)", dbname, charge_utils.ObjTypeSpace)
	space := dbmodel.NewSpacePark(dbname)
	where := map[string]interface{}{
		exist: "",
	}
	if req.ObjId > 0 {
		where["id"] = req.ObjId
	}
	reply.Total, err = space.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := space.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	spaceIds := make([]int64, len(list))
	for i, v := range list {
		spaceIds[i] = v.ID
	}

	bind := dbmodel.NewChargeBind(dbname)
	where = map[string]interface{}{
		"obj_id in":   spaceIds,
		"obj_type":    charge_utils.ObjTypeSpace,
		"charge_type": charge_utils.ChargeTypeSpace,
	}

	blist, err := bind.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	m := map[int64]int64{}

	for _, v := range blist {
		m[v.ObjId] = v.ID
	}

	bill := dbmodel.NewChargeBill(dbname)
	where = map[string]interface{}{
		"obj_id in": spaceIds,
		"obj_type":  charge_utils.ObjTypeSpace,
	}
	billList, err := bill.List(database.DB(), where, nil, -1, -1, "")
	if err != nil {
		return nil, errors.DataBaseError
	}
	bm := map[int64]bool{}
	for _, v := range billList {
		bm[v.ObjId] = true
	}

	reply.List = make([]*pb_v1.ChargeSpaceGroupItem, len(list))
	for i, v := range list {
		item := &pb_v1.ChargeSpaceGroupItem{
			SpaceNumber:  v.SpaceNumber,
			ParkNumber:   v.ParkNumber,
			ParkName:     v.ParkName,
			SpaceId:      v.ID,
			SpaceArea:    v.SpaceArea,
			SpaceStatus:  v.BindStatus,
			SpaceType:    v.SpaceType,
			HasUnpayBill: bm[v.ID],
			SpaceBindId:  m[v.ID],
		}
		reply.List[i] = item
	}

	return reply, nil
}
