// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/impl/v1/charge_utils"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkChargeSpaceNotBindListParam(req *pb_v1.ChargeSpaceNotBindListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.ChargeId < 1:
		return status.Error(10003, "费用项目不能为空")
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	return nil

}


func ChargeSpaceNotBindList(ctx context.Context, req *pb_v1.ChargeSpaceNotBindListRequest) (reply *pb_v1.ChargeSpaceNotBindListReply, err error) {

	reply = &pb_v1.ChargeSpaceNotBindListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeSpaceNotBindListParam(req)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	dbname := utils.GetGardenDbName(req.GardenId)

	chargeInfo := dbmodel.NewChargeConf(dbname)
	filter := map[string]interface{}{
		"id":req.ChargeId,
	}
	err = chargeInfo.Find(database.DB(), filter)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if chargeInfo.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	space := dbmodel.NewSpacePark(dbname)
	where := map[string]interface{}{}

	notExist :=  fmt.Sprintf("not exists(select obj_id from %s.t_charge_bind where t_charge_bind.obj_id=t_space_park.id and t_charge_bind.charge_type=? && t_charge_bind.obj_type = %d)", dbname, charge_utils.ObjTypeSpace)
	where[notExist] = chargeInfo.ChargeType
	where["house_id >"] = 0

	reply.Total, err = space.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := space.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.ChargeSpaceData, len(list))
	for i, v := range list {
		item := &pb_v1.ChargeSpaceData{
			SpaceNumber:v.SpaceNumber,
			ParkNumber:v.ParkNumber,
			ParkName:v.ParkName,
			SpaceId:v.ID,
			SpaceArea:v.SpaceArea,
			SpaceStatus:v.BindStatus,
			SpaceType:v.SpaceType,
		}
		reply.List[i] = item
	}

	return reply, nil
}
