// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkChargeVehicleBindedListParam(req *pb_v1.ChargeVehicleBindedListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.ChargeId < 1:
		return status.Error(10003, "费用项目不能为空")
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	return nil

}

func ChargeVehicleBindedList(ctx context.Context, req *pb_v1.ChargeVehicleBindedListRequest) (reply *pb_v1.ChargeVehicleBindedListReply, err error) {

	reply = &pb_v1.ChargeVehicleBindedListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeVehicleBindedListParam(req)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	dbname := utils.GetGardenDbName(req.GardenId)

	exist := fmt.Sprintf("exists(select obj_id from %s.t_charge_bind where t_charge_bind.obj_id=t_vehicle.id and t_charge_bind.charge_id=? and t_charge_bind.obj_type = %d)", dbname, charge_utils.ObjTypeVehicle)
	vehicle := dbmodel.NewVehicle(dbname)
	where := map[string]interface{}{
		exist: req.ChargeId,
	}
	reply.Total, err = vehicle.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := vehicle.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.ChargeVehicleData, len(list))
	ids := make([]int64, len(list))
	for i, v := range list {
		ids[i] = v.ID
	}
	bind := dbmodel.NewChargeBind(dbname)
	where = map[string]interface{}{
		"obj_id in": ids,
		"obj_type":  charge_utils.ObjTypeVehicle,
		"charge_id": req.ChargeId,
	}

	blist, _ := bind.List(database.DB(), where, nil, -1, -1)
	confIds := []int64{}

	m := map[int64]dbmodel.TChargeBind{}
	for _, v := range blist {
		m[v.ObjId] = v
		confIds = append(confIds, v.ChargeId)
	}

	cconf := dbmodel.NewChargeConf(dbname)
	where = map[string]interface{}{
		"id in": confIds,
	}
	cconfM := map[int64]dbmodel.TChargeConf{}
	if len(confIds) > 0 {
		li, err := cconf.List(database.DB(), where, nil, -1, -1)
		if err != nil {
			return nil, errors.DataBaseError
		}
		for _, v := range li {
			cconfM[v.ID] = v
		}
	}

	for i, v := range list {
		item := &pb_v1.ChargeVehicleData{
			VehicleType:  v.VehicleType,
			PlateNo:      v.PlateNo,
			VehicleId:    v.ID,
			Brand:        v.Brand,
			Displacement: fmt.Sprintf("%v%s", v.Displacement, v.DisplacementUnit),
			Seat:         v.Seat,
			Start:        m[v.ID].Start,
			End:          m[v.ID].End,
		}
		confId := m[v.ID].ChargeId
		if cconfM[confId].ChargeBasis == charge_utils.ChargeBasisSelf {
			item.IsCustom = true
			item.CustomFee = m[v.ID].CustomFee
		}

		reply.List[i] = item
	}

	return reply, nil
}
