// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkChargeVehicleGroupParam(req *pb_v1.ChargeVehicleGroupRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")

	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	return nil

}


func ChargeVehicleGroup(ctx context.Context, req *pb_v1.ChargeVehicleGroupRequest) (reply *pb_v1.ChargeVehicleGroupReply, err error) {

	reply = &pb_v1.ChargeVehicleGroupReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeVehicleGroupParam(req)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	dbname := utils.GetGardenDbName(req.GardenId)

	exist := fmt.Sprintf("exists(select obj_id from %s.t_charge_bind where t_charge_bind.obj_id=t_vehicle.id  and t_charge_bind.obj_type = %d)", dbname, charge_utils.ObjTypeVehicle)
	vehicle := dbmodel.NewVehicle(dbname)
	where := map[string]interface{}{
		exist:"",
	}
	if req.ObjId > 0 {
		where["id"] = req.ObjId
	}
	reply.Total, err = vehicle.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := vehicle.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.ChargeVehicleGroupItem, len(list))
	for i, v := range list {
		item := &pb_v1.ChargeVehicleGroupItem{
			VehicleType:v.VehicleType,
			PlateNo:v.PlateNo,
			VehicleId:v.ID,
			Brand:v.Brand,
			Displacement:fmt.Sprintf("%v%s", v.Displacement, v.DisplacementUnit),
			Seat:v.Seat,
		}
		reply.List[i] = item
	}

	return reply, nil
}
