// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge_conf

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)


func checkChargeConfDelParam(req *pb_v1.ChargeConfDelRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	}
	return nil

}


func ChargeConfDel(ctx context.Context, req *pb_v1.ChargeConfDelRequest) (reply *pb_v1.ChargeConfDelReply, err error) {

	reply = &pb_v1.ChargeConfDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeConfDelParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewChargeConf(dbname)
	where := map[string]interface{}{
		"id":req.Id,
		//"garden_id":req.GardenId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}


	reply.Origin = &pb_v1.ChargeConfUpdateRequest{
		GardenId:req.GardenId,
		ChargeType:old.ChargeType,
		ChargeName:old.ChargeName,
		ChargeTimeType:old.ChargeTimeType,
		ChargeBasis:old.ChargeBasis,
		FixAmount:old.FixAmount,
		FixAmountName:old.FixAmountName,
		UnitPrice:old.UnitPrice,
		ChargeEffectiveTime:old.ChargeEffectiveTime,
		BillPeriod:old.BillPeriod,
		BillPeriodType:old.BillPeriodType,
		LateFeeEnable:false,
		LateFeeMaxDays:old.LateFeeMaxDays,
		LateFeeDay:old.LateFeeDay,
		LateFeePercent:old.LateFeePercent,
		Id:req.Id,
	}
	if old.LateFeeEnable == 1 {
		reply.Origin.LateFeeEnable = true
	}
	// 删除数据
	chargeConf := dbmodel.NewChargeConf(dbname)
	db := database.DB().Begin()

	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()

	err = chargeConf.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	isBind, err := charge_utils.IsChargeBindObj(req.Id, dbname)
	if err != nil {
		return nil, err
	}
	if isBind {
		return nil, status.Error(10003, "操作失败，已绑定对象")
	}

	db.Commit()

	return reply, nil
}
