// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge_conf

import (
	"context"
	"encoding/json"
	"fmt"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkChargeConfListParam(req *pb_v1.ChargeConfListRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == -1 || req.Page == -1 {
		req.Page = -1
		req.PageSize = -1
	}

	return nil

}

func ChargeConfList(ctx context.Context, req *pb_v1.ChargeConfListRequest) (reply *pb_v1.ChargeConfListReply, err error) {

	reply = &pb_v1.ChargeConfListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkChargeConfListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	p := dbmodel.NewChargeConf(dbname)
	where := map[string]interface{}{
		//"garden_id":req.GardenId,
	}
	if req.ChargeType > 0 {
		where["charge_type"] = req.ChargeType
	}
	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.ChargeConfItem, len(list))
	for i, v := range list {

		item := &pb_v1.ChargeConfItem{
			GardenId:            req.GardenId,
			ChargeType:          v.ChargeType,
			ChargeName:          v.ChargeName,
			ChargeTimeType:      v.ChargeTimeType,
			ChargeBasis:         v.ChargeBasis,
			FixAmount:           v.FixAmount,
			FixAmountName:       v.FixAmountName,
			UnitPrice:           v.UnitPrice,
			Id:                  v.ID,
			ChargeEffectiveTime: v.ChargeEffectiveTime,
			BillPeriod:          v.BillPeriod,
			BillPeriodType:      v.BillPeriodType,
			LateFeeEnable:       false,
			LateFeeMaxDays:      v.LateFeeMaxDays,
			LateFeeDay:          v.LateFeeDay,
			LateFeePercent:      v.LateFeePercent,
		}
		if v.LateFeeEnable == 1 {
			item.LateFeeEnable = true
		}
		reply.List[i] = item
	}

	return reply, nil
}
