// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge_power_record

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/charge"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"
)

func checkPowerRecordAddParam(req *pb_v1.PowerRecordAddRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.HouseId == 0:
		return status.Error(10003, "房屋不能为空")
	case charge_utils.IsPowerChargeType[req.ChargeType] == false:
		return status.Error(10003, "费用类型错误")
	case req.Start == 0:
		return status.Error(10003, "开始时间不能为空")
	case req.End == 0:
		return status.Error(10003, "结束时间不能为空")
	case req.End <= req.Start:
		return status.Error(10003, "结束时间必须晚于开始时间")
	case req.LastAmount == 0 || req.CurrentAmount == 0:
		return status.Error(10003, "使用量不能为空")
	case req.LastAmount > req.CurrentAmount:
		return status.Error(10003, "使用量错误")
	}

	return nil

}

func getHouseInfos(ids []int64, dbname string) (map[int64]dbmodel.THouseUnitBuilding, error) {
	ret := map[int64]dbmodel.THouseUnitBuilding{}
	if len(ids) == 0 {
		return ret, nil
	}
	p := dbmodel.NewHouseUnitBuilding(dbname)
	where := map[string]interface{}{
		"id in": ids,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}

	for _, v := range list {
		ret[v.ID] = v
	}
	return ret, nil
}

func PowerRecordAdd(ctx context.Context, req *pb_v1.PowerRecordAddRequest) (reply *pb_v1.PowerRecordAddReply, err error) {

	reply = &pb_v1.PowerRecordAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkPowerRecordAddParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)

	house := dbmodel.NewHouseUnitBuilding(dbname)
	where := map[string]interface{}{
		"id": req.HouseId,
	}
	err = house.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if house.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	now := time.Now()
	power := &dbmodel.TChargePower{
		Start:         req.Start,
		End:           req.End,
		LastAmount:    req.LastAmount,
		CurrentAmount: req.CurrentAmount,
		ChargeType:    req.ChargeType,
		HouseId:       req.HouseId,
		Comment:       req.Comment,
		CreatedAt:     now,
		UpdatedAt:     now,
	}
	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	power.SetTable(dbname)
	err = power.Insert(db)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			return nil, status.Error(10003, "存在重复记录")
		}
		return nil, errors.DataBaseError
	}

	bind := dbmodel.NewChargeBind(dbname)
	where = map[string]interface{}{
		"obj_id":      req.HouseId,
		"charge_type": req.ChargeType,
	}

	err = bind.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if bind.ID == 0 {
		return nil, status.Error(10003, "未绑定费项")
	}
	// 没有使用量 都不生成账单
	if req.LastAmount == req.CurrentAmount {
		db.Commit()
		return reply, nil
	}

	chargeInfo := dbmodel.NewChargeConf(dbname)
	where = map[string]interface{}{
		"id": bind.ChargeId,
		//"garden_id": req.GardenId,
	}
	err = chargeInfo.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if chargeInfo.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	param := charge.BillAddParam{
		HouseInfo:   house,
		PowerInfo:   power,
		VehicleInfo: nil,
		SpaceInfo:   nil,
		ChargeInfo:  chargeInfo,
		BindInfo:    bind,
		ObjType:     charge_utils.ObjTypeHouse,
	}
	err = charge.BillAddHousePower(&param, db, dbname)
	if err != nil {
		return nil, err
	}
	db.Commit()

	return reply, nil
}
