// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge_power_record

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/charge"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"
)

func checkPowerRecordBatchAddParam(req *pb_v1.PowerRecordBatchAddRequest) (buidings []string, units map[string][]int64, houses map[string][]string, records map[string][]*pb_v1.PowerRecordBatchData, err error) {
	switch {
	case req.GardenId < 1:
		return nil, nil, nil, nil, status.Error(10003, "小区不能为空")
	case charge_utils.IsPowerChargeType[req.ChargeType] == false:
		return nil, nil, nil, nil, status.Error(10003, "费用类型不能为空")
	}
	units = map[string][]int64{}
	houses = map[string][]string{}
	records = map[string][]*pb_v1.PowerRecordBatchData{}
	for _, v := range req.List {
		switch {
		case v.HouseNumber == "":
			return nil, nil, nil, nil, status.Error(10003, "房屋不能为空")
		case v.Start == 0:
			return nil, nil, nil, nil, status.Error(10003, "开始时间不能为空")
		case v.End == 0:
			return nil, nil, nil, nil, status.Error(10003, "结束时间不能为空")
		case v.End <= v.Start:
			return nil, nil, nil, nil, status.Error(10003, "结束时间必须晚于开始时间")
		case v.LastAmount == 0 || v.CurrentAmount == 0:
			return nil, nil, nil, nil, status.Error(10003, "使用量不能为空")
		case v.LastAmount > v.CurrentAmount:
			return nil, nil, nil, nil, status.Error(10003, "使用量错误")
		case v.BuildingNumber == "":
			return nil, nil, nil, nil, status.Error(10003, "楼栋编号不能为空")
		case v.UnitNumber < 1:
			return nil, nil, nil, nil, status.Error(10003, "单元号不能为空")
		}

		if _, ok := units[v.BuildingNumber]; !ok {
			buidings = append(buidings, v.BuildingNumber)
			units[v.BuildingNumber] = []int64{}
		}
		buidingUnit := fmt.Sprintf("%s-%d", v.BuildingNumber, v.UnitNumber)
		buidingUnitHouse := fmt.Sprintf("%s-%d-%s", v.BuildingNumber, v.UnitNumber, v.HouseNumber)

		if _, ok := houses[buidingUnit]; !ok {
			units[v.BuildingNumber] = append(units[v.BuildingNumber], v.UnitNumber)
			houses[buidingUnit] = []string{}
		}

		if _, ok := records[buidingUnitHouse]; !ok {
			records[buidingUnitHouse] = []*pb_v1.PowerRecordBatchData{}
			houses[buidingUnit] = append(houses[buidingUnit], v.HouseNumber)
		}
		records[buidingUnitHouse] = append(records[buidingUnitHouse], v)
	}

	return
}

func getBuildings(buildingNumbers []string, dbname string) (map[string]dbmodel.TBuilding, error) {
	p := dbmodel.NewBuilding(dbname)
	where := map[string]interface{}{
		"building_number in": buildingNumbers,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if len(list) != len(buildingNumbers) {
		return nil, status.Error(10003, "请在系统录入相应的楼栋")
	}
	m := map[string]dbmodel.TBuilding{}
	for _, v := range list {
		m[v.BuildingNumber] = v

	}
	return m, nil
}

func getUnits(buildingNumberM map[string]dbmodel.TBuilding, units map[string][]int64, dbname string) (map[string]dbmodel.TUnit, error) {
	ret := map[string]dbmodel.TUnit{}
	for buildNumber, unitArray := range units {
		p := dbmodel.NewUnit(dbname)
		where := map[string]interface{}{
			"unit_number in": unitArray,
			"building_id":    buildingNumberM[buildNumber].ID,
		}
		list, err := p.List(database.DB(), where, nil, -1, -1)
		if err != nil {
			return nil, errors.DataBaseError
		}
		if len(list) != len(unitArray) {
			return nil, status.Error(10003, "请在系统录入相应的单元")
		}
		for _, v := range list {
			ret[buildNumber+"-"+fmt.Sprintf("%d", v.UnitNumber)] = v
		}
	}

	return ret, nil
}

func getHouses(buildingUnitM map[string]dbmodel.TUnit, houses map[string][]string, dbname string) (map[string][]dbmodel.THouseUnitBuilding, error) {
	ret := map[string][]dbmodel.THouseUnitBuilding{}
	for buildAndUnitNumber, houseArray := range houses {
		p := dbmodel.NewHouseUnitBuilding(dbname)
		where := map[string]interface{}{
			"house_number in": houseArray,
			"building_id":     buildingUnitM[buildAndUnitNumber].BuildingId,
			"unit_id":         buildingUnitM[buildAndUnitNumber].ID,
		}
		list, err := p.List(database.DB(), where, nil, -1, -1)
		if err != nil {
			return nil, errors.DataBaseError
		}
		if len(list) != len(houseArray) {
			return nil, status.Error(10003, "请在系统录入相应的房屋")
		}

		ret[buildAndUnitNumber] = list
	}

	return ret, nil
}

func getOneUnitHouses(buildingUnit string, houseArray []dbmodel.THouseUnitBuilding, records map[string][]*pb_v1.PowerRecordBatchData) (houseM map[int64]dbmodel.THouseUnitBuilding, recordsByHouseId map[int64][]*pb_v1.PowerRecordBatchData) {
	recordsByHouseId = map[int64][]*pb_v1.PowerRecordBatchData{}
	houseM = map[int64]dbmodel.THouseUnitBuilding{}
	for _, house := range houseArray {
		houseM[house.ID] = house
		recordsByHouseId[house.ID] = records[fmt.Sprintf("%s-%s", buildingUnit, house.HouseNumber)]
	}
	return
}

func getPowerChargeInfo(chargeIds []int64, chargeType int32, dbname string) (map[int64]dbmodel.TChargeConf, error) {
	ret := map[int64]dbmodel.TChargeConf{}
	p := dbmodel.NewChargeConf(dbname)
	where := map[string]interface{}{
		"id in":       chargeIds,
		"charge_type": chargeType,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range list {
		ret[v.ID] = v
	}
	return ret, nil
}

func getPowerBindInfo(houseIds []int64, chargeType int32, dbname string) (map[int64]dbmodel.TChargeBind, map[int64]dbmodel.TChargeConf, error) {
	ret := map[int64]dbmodel.TChargeBind{}
	p := dbmodel.NewChargeBind(dbname)
	where := map[string]interface{}{
		"obj_id in":   houseIds,
		"charge_type": chargeType,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, nil, errors.DataBaseError
	}
	chargeIds := []int64{}
	chargeIdsM := map[int64]bool{}
	for _, v := range list {
		ret[v.ObjId] = v
		if !chargeIdsM[v.ChargeId] {
			chargeIdsM[v.ChargeId] = true
			chargeIds = append(chargeIds, v.ChargeId)
		}
	}
	chargeIdsM = nil

	chargeInfoM, err := getPowerChargeInfo(chargeIds, chargeType, dbname)
	if err != nil {
		return nil, nil, err
	}

	return ret, chargeInfoM, nil
}

func batchAdd(db *gorm.DB, req *pb_v1.PowerRecordBatchAddRequest, houseInfos map[int64]dbmodel.THouseUnitBuilding, batchData map[int64][]*pb_v1.PowerRecordBatchData, dbname string) error {

	now := time.Now()
	array := []dbmodel.TChargePower{}
	houseIds := []int64{}
	for houseId, records := range batchData {
		houseIds = append(houseIds, houseId)
		for _, record := range records {
			// 抄表记录
			power := dbmodel.TChargePower{
				Start:         record.Start,
				End:           record.End,
				LastAmount:    record.LastAmount,
				CurrentAmount: record.CurrentAmount,
				ChargeType:    req.ChargeType,
				//GardenId:      req.GardenId,
				HouseId:   houseId,
				Comment:   record.Comment,
				CreatedAt: now,
				UpdatedAt: now,
			}
			power.SetTable(dbname)
			array = append(array, power)
		}
	}
	power := dbmodel.NewChargePower(dbname)
	err := power.InsertMulti(db, &array)
	if err != nil {
		return errors.DataBaseError
	}

	bindM, chargeInfoM, err := getPowerBindInfo(houseIds, req.ChargeType, dbname)
	if err != nil {
		return err
	}

	for _, v := range array {

		if bindM[v.HouseId].ID == 0 {
			return status.Error(10003, fmt.Sprintf("房屋%v-%v-%v未绑定费项", houseInfos[v.HouseId].BuildingNumber, houseInfos[v.HouseId].UnitNumber, houseInfos[v.HouseId].HouseNumber))
		}
		if chargeInfoM[bindM[v.HouseId].ChargeId].ID == 0 {
			return status.Error(10003, fmt.Sprintf("房屋%v-%v-%v绑定的费项不存在", houseInfos[v.HouseId].BuildingNumber, houseInfos[v.HouseId].UnitNumber, houseInfos[v.HouseId].HouseNumber))
		}
		if v.LastAmount == v.CurrentAmount {
			continue
		}

		houseInfo := houseInfos[v.HouseId]
		bindInfo := bindM[v.HouseId]
		chargeInfo := chargeInfoM[bindInfo.ChargeId]
		powerInfo := v
		param := charge.BillAddParam{
			HouseInfo:   &houseInfo,
			PowerInfo:   &powerInfo,
			VehicleInfo: nil,
			SpaceInfo:   nil,
			ChargeInfo:  &chargeInfo,
			BindInfo:    &bindInfo,
			ObjType:     charge_utils.ObjTypeHouse,
		}
		err = charge.BillAddHousePower(&param, db, dbname)
		if err != nil {
			return err
		}
	}

	return nil
}

func PowerRecordBatchAdd(ctx context.Context, req *pb_v1.PowerRecordBatchAddRequest) (reply *pb_v1.PowerRecordBatchAddReply, err error) {

	reply = &pb_v1.PowerRecordBatchAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	sBuildings, sUnits, sHouses, sRecords, err := checkPowerRecordBatchAddParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)

	// 获取楼栋id
	buildings, err := getBuildings(sBuildings, dbname)
	if err != nil {
		return nil, err
	}

	// 获取楼栋下得单元id
	units, err := getUnits(buildings, sUnits, dbname)
	if err != nil {
		return nil, err
	}
	// 获取楼栋+单元下的房屋id
	houses, err := getHouses(units, sHouses, dbname)
	if err != nil {
		return nil, err
	}

	// 一个单元一个单元录入
	db := database.DB().Begin()
	for buidingUnit, houseArray := range houses {
		houseM, records := getOneUnitHouses(buidingUnit, houseArray, sRecords)
		err = batchAdd(db, req, houseM, records, dbname)
		if err != nil {
			db.Rollback()
			return nil, err
		}
	}
	db.Commit()

	return reply, nil
}
