// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge_power_record

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkPowerRecordDelParam(req *pb_v1.PowerRecordDelRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	}

	return nil

}

func PowerRecordDel(ctx context.Context, req *pb_v1.PowerRecordDelRequest) (reply *pb_v1.PowerRecordDelReply, err error) {

	reply = &pb_v1.PowerRecordDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkPowerRecordDelParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	// 原始记录
	old := dbmodel.NewChargePower(dbname)
	where := map[string]interface{}{
		"id": req.Id,
		//"garden_id": req.GardenId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.PowerRecordUpdateRequest{
		Start:         old.Start,
		End:           old.End,
		Id:            old.ID,
		LastAmount:    old.LastAmount,
		CurrentAmount: old.CurrentAmount,
		ChargeType:    old.ChargeType,
		GardenId:      req.GardenId,
		HouseId:       old.HouseId,
		Comment:       old.Comment,
		//ReadAt:        old.ReadAt,
	}

	// 更新数据库
	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()

	where = map[string]interface{}{
		"id": req.Id,
	}
	err = old.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	bill := dbmodel.NewChargeBill(dbname)
	where = map[string]interface{}{
		"charge_power_id": req.Id,
	}
	err = bill.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	db.Commit()

	return reply, nil

}
