// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package charge_power_record

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkPowerRecordListParam(req *pb_v1.PowerRecordListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	return nil

}

func getChargeInfos(ids []int64, dbname string) (map[int64]dbmodel.TChargeConf, error) {
	ret := map[int64]dbmodel.TChargeConf{}
	if len(ids) == 0 {
		return ret, nil
	}
	p := dbmodel.NewChargeConf(dbname)
	where := map[string]interface{}{
		"id in": ids,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}

	for _, v := range list {
		ret[v.ID] = v
	}
	return ret, nil
}

func PowerRecordList(ctx context.Context, req *pb_v1.PowerRecordListRequest) (reply *pb_v1.PowerRecordListReply, err error) {

	reply = &pb_v1.PowerRecordListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkPowerRecordListParam(req)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	dbname := utils.GetGardenDbName(req.GardenId)

	where := map[string]interface{}{}
	if req.ChargeType > 0 {
		where["charge_type"] = req.ChargeType
	}

	power := dbmodel.NewChargePower(dbname)

	count, err := power.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if count == 0 {
		return reply, nil
	}
	reply.Total = count
	list, err := power.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.PowerRecordItem, len(list))
	houseIds := []int64{}
	for _, v := range list {
		houseIds = append(houseIds, v.HouseId)
	}

	houseInfos, err := getHouseInfos(houseIds, dbname)
	if err != nil {
		return nil, err
	}
	for i, v := range list {
		item := &pb_v1.PowerRecordItem{
			Id:            v.ID,
			ChargeType:    v.ChargeType,
			LastAmount:    v.LastAmount,
			CurrentAmount: v.CurrentAmount,
			Start:         v.Start,
			End:           v.End,
			Comment:       v.Comment,
			HouseId:       v.HouseId,
			CreatedAt:     v.CreatedAt.Unix(),
			HouseName: fmt.Sprintf("%v-%v-%v",
				houseInfos[v.HouseId].BuildingNumber,
				houseInfos[v.HouseId].UnitNumber,
				houseInfos[v.HouseId].HouseNumber),
		}
		if v.BillId > 0 {
			item.HasBill = true
		}
		reply.List[i] = item
	}

	return reply, nil
}
