// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package event

import (
	"context"
	"encoding/json"
	"fmt"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkEventAddParam(req *pb_v1.EventAddRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Title == "":
		return status.Error(10003, "标题不能为空")
	case req.Content == "":
		return status.Error(10003, "内容不能为空")
	case req.Start < 1:
		return status.Error(10003, "活动开始时间不能为空")
	case req.End < 1:
		return status.Error(10003, "活动结束时间不能为空")
	case req.End < req.Start:
		return status.Error(10003, "结束时间不能晚于开始时间")
	case req.SignUpEnd < 1:
		return status.Error(10003, "报名截止时间不能为空")
	case req.SignUpEnd > req.End:
		return status.Error(10003, "报名截止时间不能晚于活动结束时间")
	case req.PeopleLimit < 1:
		return status.Error(10003, "人数限制不能为空")
	case req.Addr == "":
		return status.Error(10003, "活动地址不能为空")

	}

	pic := []string{}
	for _, v := range req.Pic {
		if oss_utils.IsFix(v) {
			continue
		}
		pic = append(pic, v)
	}
	req.Pic = pic

	return nil

}

//
func EventAdd(ctx context.Context, req *pb_v1.EventAddRequest) (reply *pb_v1.EventAddReply, err error) {

	reply = &pb_v1.EventAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkEventAddParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)

	now := time.Now()
	p := &dbmodel.TEvent{
		Title:       req.Title,
		Content:     req.Content,
		Start:       req.Start,
		End:         req.End,
		SignUpEnd:   req.SignUpEnd,
		PeopleLimit: req.PeopleLimit,
		Addr:        req.Addr,
		Fee:         req.Fee,
		UpdatedAt:   now,
		CreatedAt:   now,
		//GardenId:req.GardenId,
	}
	if len(req.Pic) > 0 {
		p.Pic = utils.StringJoin(req.Pic, ";")
	}
	p.SetTable(dbname)

	db := database.DB().Begin()
	err = p.Insert(db)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	inList := oss_utils.GetObjFromText(req.Content)
	if len(req.Pic) > 0 {
		inList = append(inList, req.Pic...)
	}
	if len(inList) > 0 {
		if err := oss_utils.OssObjAdd(inList, nil); err != nil {
			db.Rollback()
			return nil, err
		}
	}

	db.Commit()

	reply.Id = p.ID

	return reply, nil
}
