// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/cache"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/statistic"
	dbmodel "property-garden/model"
	"property-garden/pb"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkBuildingBatchParam(req *pb_v1.BatchBuildingItem) error {
	switch {
	case req.BuildingNumber == "":
		return status.Error(10003, "楼栋编号不能为空")
	case strings.Contains(req.BuildingNumber, "-"):
		return status.Error(10003, "楼栋号不能包含-")
	case req.BuildingName == "":
		return status.Error(10003, "楼栋名称不能为空")

	}
	return nil
}

func checkHouseCount(gardenId int64, count int64) error {
	if count == 0 {
		return nil
	}
	key := fmt.Sprintf("house_count_check_%d", gardenId)
	ok := cache.RedisLock(key)
	if !ok {
		return status.Error(10001, "系统繁忙")
	}
	defer cache.RedisUnlock(key)

	mreq := pb_v1.GardenHouseCountLimitRequest{GardenId: gardenId}
	mreply, err := pb.System.GardenHouseCountLimit(context.Background(), &mreq)
	if err != nil {
		return err
	}
	if mreply.HouseCount+count > mreply.HouseCountLimit {
		return status.Error(10003, "房屋数量超过套餐限制")
	}
	return nil
}

func houseCountChange(gardenId int64, count int64) error {
	if count == 0 {
		return nil
	}
	mreq := pb_v1.GardenHouseCountChangeRequest{GardenId: gardenId, Count: count}
	_, err := pb.System.GardenHouseCountChange(context.Background(), &mreq)
	if err != nil {
		return err
	}
	return nil
}

func checkUnitBatchParam(req *pb_v1.BatchUnitItem) error {
	switch {
	case req.UnitNumber < 0:
		return status.Error(10003, "单元编号不能为空")
	case req.UnitName == "":
		return status.Error(10003, "单元名称名称不能为空")
	case req.UnitLayers < 1:
		return status.Error(10003, "层数不能为空")
	case req.BuildingNumber == "":
		return status.Error(10003, "楼栋编码不能为空")
	case strings.Contains(req.BuildingNumber, "-"):
		return status.Error(10003, "楼栋号不能包含-")

	}
	return nil
}

func checkHouseBatchParam(req *pb_v1.BatchHouseItem) error {
	switch {
	case req.HouseNumber == "":
		return status.Error(10003, "门牌号不能为空")
	case strings.Contains(req.HouseNumber, "-"):
		return status.Error(10003, "门牌号不能包含-")
	case req.Layer < 1:
		return status.Error(10003, "楼层不能为空")
	case req.BuildingNumber == "":
		return status.Error(10003, "楼栋编号不能为空")
	case req.RoomCount < 0 || req.HallCount < 0:
		return status.Error(10003, "几室几厅不能为空")
	case req.HouseType < 1:
		return status.Error(10003, "房屋类型不能为空")
	case req.UnitNumber < 0:
		return status.Error(10003, "单元编号不能为空")
	case req.HouseArea < 1 || req.HouseUsedArea < 1:
		return status.Error(10003, "建筑面积和使用面积不能为空")
	case strings.Contains(req.BuildingNumber, "-"):
		return status.Error(10003, "楼栋号不能包含-")

	}
	return nil
}

func getBuildingNumberMap(numbers []string, db *gorm.DB, dbname string) (numberMap map[string]dbmodel.TBuilding, err error) {
	numberMap = map[string]dbmodel.TBuilding{}

	p := dbmodel.NewBuilding(dbname)
	where := map[string]interface{}{
		"building_number in": numbers,
	}
	list, err := p.List(db, where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range list {
		numberMap[v.BuildingNumber] = v
	}
	return numberMap, nil
}

func getUnitNumberMap(unitNumbers []int64, buildingNumbers []string, db *gorm.DB, dbname string) (numberMap map[string]dbmodel.TUnit, err error) {
	buildingNumberMap, err := getBuildingNumberMap(buildingNumbers, db, dbname)
	if err != nil {
		return nil, err
	}
	buildingIdNumber := map[int64]string{}
	numberMap = map[string]dbmodel.TUnit{}

	or := map[string]interface{}{}
	// 参数检查
	for i, v := range unitNumbers {
		building, ok := buildingNumberMap[buildingNumbers[i]]
		if !ok {
			return nil, status.Error(10003, "单元所需楼栋编号不存在")
		}
		buildingIdNumber[building.ID] = buildingNumbers[i]
		or[fmt.Sprintf("unit_number=%d and building_id=%d", v, building.ID)] = ""
	}

	// 查看已存在的数据
	p := dbmodel.NewUnit(dbname)

	list, err := p.List(db, nil, or, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range list {
		numberMap[fmt.Sprintf("%d-%s", v.UnitNumber, buildingIdNumber[v.BuildingId])] = v
	}

	return numberMap, nil
}

func batchBuildingAdd(req *pb_v1.BatchHouseAddRequest, db *gorm.DB, dbname string) (err error) {
	if len(req.Buildings) == 0 {
		return nil
	}
	numberMap := map[string]dbmodel.TBuilding{}
	inArray := []string{}
	now := time.Now()
	// 参数检查
	for _, v := range req.Buildings {
		if err = checkBuildingBatchParam(v); err != nil {
			return err
		}
		inArray = append(inArray, v.BuildingNumber)
	}

	// 查看已存在的数据
	p := dbmodel.NewBuilding(dbname)
	where := map[string]interface{}{
		"building_number in": inArray,
	}
	list, err := p.List(db, where, nil, -1, -1)
	if err != nil {
		return errors.DataBaseError
	}
	for _, v := range list {
		numberMap[v.BuildingNumber] = v
	}

	array := []dbmodel.TBuilding{}
	for _, v := range req.Buildings {
		// 已存在的不再添加
		if _, ok := numberMap[v.BuildingNumber]; ok {
			continue
		}

		item := &dbmodel.TBuilding{
			//GardenId:req.GardenId,
			BuildingName:     v.BuildingName,
			BuildingNumber:   v.BuildingNumber,
			BuildingArea:     v.BuildingArea,
			BuildingUsedArea: v.BuildingUsedArea,
			Comment:          v.Comment,
			UpdatedAt:        now,
			CreatedAt:        now,
		}
		item.SetTable(dbname)
		array = append(array, *item)
	}
	if len(array) == 0 {
		return nil
	}
	// 批量添加
	err = p.InsertMulti(db, &array)
	if err != nil {
		return errors.DataBaseError
	}

	return nil
}

func getBuildingNumbersForUnit(req *pb_v1.BatchHouseAddRequest) []string {
	ret := []string{}
	m := map[string]bool{}
	for _, v := range req.Units {
		if m[v.BuildingNumber] {
			continue
		}
		ret = append(ret, v.BuildingNumber)
		m[v.BuildingNumber] = true
	}
	return ret
}

func getUnitBuildingNumbersForHouse(req *pb_v1.BatchHouseAddRequest) ([]int64, []string) {
	unitNumber := []int64{}
	buildingNumber := []string{}
	m := map[int64]bool{}
	for _, v := range req.Houses {
		if m[v.UnitNumber] {
			continue
		}
		unitNumber = append(unitNumber, v.UnitNumber)
		buildingNumber = append(buildingNumber, v.BuildingNumber)
		m[v.UnitNumber] = true
	}
	return unitNumber, buildingNumber
}

func batchUnitAdd(req *pb_v1.BatchHouseAddRequest, db *gorm.DB, dbname string) (err error) {
	if len(req.Units) == 0 {
		return nil
	}
	buildingNumbers := getBuildingNumbersForUnit(req)
	buildingNumberMap, err := getBuildingNumberMap(buildingNumbers, db, dbname)
	if err != nil {
		return err
	}

	buildingIdNumber := map[int64]string{}
	numberMap := map[string]dbmodel.TUnit{}

	or := map[string]interface{}{}
	// 参数检查
	for _, v := range req.Units {
		if err = checkUnitBatchParam(v); err != nil {
			return err
		}
		building, ok := buildingNumberMap[v.BuildingNumber]
		if !ok {
			return status.Error(10003, "单元所需楼栋编号不存在")
		}
		buildingIdNumber[building.ID] = v.BuildingNumber
		or[fmt.Sprintf("unit_number=%d and building_id=%d", v.UnitNumber, building.ID)] = ""
	}

	// 查看已存在的数据
	p := dbmodel.NewUnit(dbname)
	now := time.Now()

	list, err := p.List(db, nil, or, -1, -1)
	if err != nil {
		return errors.DataBaseError
	}
	for _, v := range list {
		numberMap[fmt.Sprintf("%d-%s", v.UnitNumber, buildingIdNumber[v.BuildingId])] = v
	}

	array := []dbmodel.TUnit{}
	for _, v := range req.Units {
		// 已存在的不再添加
		if _, ok := numberMap[fmt.Sprintf("%d-%s", v.UnitNumber, v.BuildingNumber)]; ok {
			continue
		}

		hasLift := int64(2)
		if v.HasLift {
			hasLift = 1
		}
		item := &dbmodel.TUnit{
			//GardenId:req.GardenId,
			UnitName:   v.UnitName,
			UnitNumber: v.UnitNumber,
			UnitLayers: v.UnitLayers,
			BuildingId: buildingNumberMap[v.BuildingNumber].ID,
			HasLift:    hasLift,
			UpdatedAt:  now,
			CreatedAt:  now,
		}
		item.SetTable(dbname)
		array = append(array, *item)
	}
	if len(array) == 0 {
		return nil
	}
	// 批量添加
	err = p.InsertMulti(db, &array)
	if err != nil {
		return errors.DataBaseError
	}

	return nil
}

func batchHouseAdd(req *pb_v1.BatchHouseAddRequest, unitAndBuildingNumberMap map[string]dbmodel.TUnit, db *gorm.DB, dbname string) (insertCount int, err error) {

	buildingIdNumber := map[int64]string{}
	unitIdNumber := map[int64]int64{}
	or := map[string]interface{}{}
	// 参数检查
	for _, v := range req.Houses {
		if err = checkHouseBatchParam(v); err != nil {
			return 0, err
		}
		unit, ok := unitAndBuildingNumberMap[fmt.Sprintf("%d-%s", v.UnitNumber, v.BuildingNumber)]
		if !ok {
			return 0, status.Error(10003, "房屋所需楼栋或单元不存在")
		}
		buildingIdNumber[unit.BuildingId] = v.BuildingNumber
		unitIdNumber[unit.ID] = v.UnitNumber
		or[fmt.Sprintf("house_number = %s and unit_id=%d and building_id=%d", v.HouseNumber, unit.ID, unit.BuildingId)] = ""
	}

	// 查看已存在的数据
	p := dbmodel.NewHouse(dbname)
	now := time.Now()
	where := map[string]interface{}{
		//"garden_id":req.GardenId,
	}

	list, err := p.List(db, where, or, -1, -1)
	if err != nil {
		return 0, errors.DataBaseError
	}
	houseNumberMap := map[string]bool{}
	for _, v := range list {
		houseNumberMap[fmt.Sprintf("%s-%d-%s", v.HouseNumber, unitIdNumber[v.UnitId], buildingIdNumber[v.BuildingId])] = true
	}

	array := []dbmodel.THouse{}
	for _, v := range req.Houses {
		// 已存在的不再添加
		if _, ok := houseNumberMap[fmt.Sprintf("%s-%d-%s", v.HouseNumber, v.UnitNumber, v.BuildingNumber)]; ok {
			continue
		}

		item := &dbmodel.THouse{
			//GardenId:      req.GardenId,
			HouseType:     v.HouseType,
			HouseNumber:   v.HouseNumber,
			BuildingId:    unitAndBuildingNumberMap[fmt.Sprintf("%d-%s", v.UnitNumber, v.BuildingNumber)].BuildingId,
			UnitId:        unitAndBuildingNumberMap[fmt.Sprintf("%d-%s", v.UnitNumber, v.BuildingNumber)].ID,
			HasLift:       unitAndBuildingNumberMap[fmt.Sprintf("%d-%s", v.UnitNumber, v.BuildingNumber)].HasLift,
			HouseArea:     v.HouseArea,
			HouseUsedArea: v.HouseUsedArea,
			Layer:         v.Layer,
			RoomCount:     v.RoomCount,
			HallCount:     v.HallCount,
			UpdatedAt:     now,
			CreatedAt:     now,
			Status:        HouseStatusWait,
		}
		item.SetTable(dbname)
		array = append(array, *item)
	}
	if len(array) == 0 {
		return 0, nil
	}
	insertCount += len(array)
	// 批量添加
	err = p.InsertMulti(db, &array)
	if err != nil {
		return 0, errors.DataBaseError
	}

	mreq := pb_v1.ObjStatisticSetRequest{
		ObjType:       statistic.ObjTypeHouse,
		TotalIncrease: int64(len(array)),
		GardenId:      req.GardenId,
	}
	err = statistic.ObjStatisticSetWithDb(&mreq, db)
	if err != nil {
		return 0, err
	}

	return insertCount, nil
}

//
func BatchHouseAdd(ctx context.Context, req *pb_v1.BatchHouseAddRequest) (reply *pb_v1.BatchHouseAddReply, err error) {

	reply = &pb_v1.BatchHouseAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	db := database.DB().Begin()
	dbname := utils.GetGardenDbName(req.GardenId)

	// 添加楼栋
	err = batchBuildingAdd(req, db, dbname)
	if err != nil {
		db.Rollback()
		return nil, err
	}

	// 添加单元
	err = batchUnitAdd(req, db, dbname)
	if err != nil {
		db.Rollback()
		return nil, err
	}

	if len(req.Houses) == 0 {
		db.Commit()
		return reply, nil
	}

	// 添加房屋， unitNumbers 和 buildingNumbers的下标是对应的
	unitNumbers, buildingNumbers := getUnitBuildingNumbersForHouse(req)
	unitAndBuildingNumberMap, err := getUnitNumberMap(unitNumbers, buildingNumbers, db, dbname)
	if err != nil {
		return nil, err
	}

	// 如果房屋数据量过大，分批次添加
	length := len(req.Houses)

	tmp := req.Houses[:]
	TotalCount := 0
	for i := 0; i < length; i += 100 {
		end := i + 100
		if end > length {
			end = length
		}
		req.Houses = tmp[i:end]
		inCount := 0
		inCount, err = batchHouseAdd(req, unitAndBuildingNumberMap, db, dbname)
		if err != nil {
			db.Rollback()
			return nil, err
		}
		TotalCount += inCount
	}

	if err = checkHouseCount(req.GardenId, int64(TotalCount)); err != nil {
		db.Rollback()
		return nil, err
	}
	if err = houseCountChange(req.GardenId, int64(TotalCount)); err != nil {
		db.Rollback()
		return nil, err
	}

	db.Commit()

	return reply, nil

}
