// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	"property-garden/impl/v1/statistic"
	dbmodel "property-garden/model"
	"property-garden/pb"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkBuildingDelParam(req *pb_v1.BuildingDelRequest) error {
	switch {
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "小区id不能为空")

	}
	return nil
}

const (
	HouseholdApproveStatusWait    = 1
	HouseholdApproveStatusSuccess = 2
	HouseholdApproveStatusFail    = 3
)

func RefuseHouseApply(gardenId int64, buildingId int64, unitId int64, houseId int64) error {
	mreq := pb_v1.HouseholdRefuseRequest{
		GardenId:   gardenId,
		HouseId:    houseId,
		BuildingId: buildingId,
		UnitId:     unitId,
		Feedback:   "房屋变更，请重新申请",
	}
	_, err := pb.Household.HouseholdRefuse(context.Background(), &mreq)
	if err != nil {
		return err
	}
	return nil
}

func GetHouseIds(buildingId int64, unitId int64, houseId int64, dbname string) ([]int64, error) {
	if houseId > 0 {
		return []int64{houseId}, nil
	}
	where := map[string]interface{}{}
	if unitId > 0 {
		where["unit_id"] = unitId
	}
	if buildingId > 0 {
		where["building_id"] = buildingId
	}
	p := dbmodel.NewHouse(dbname)
	ids, err := p.GetIds(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	return ids, nil
}

func IsHouseBindCharge(gardenId int64, houseIds []int64, db *gorm.DB) (bool, error) {
	dbname := utils.GetGardenDbName(gardenId)
	p := dbmodel.NewChargeBind(dbname)
	where := map[string]interface{}{
		"obj_id in": houseIds,
		"obj_type":  charge_utils.ObjTypeHouse,
	}
	count, err := p.Count(db, where, nil)
	if err != nil {
		return false, errors.DataBaseError
	}
	if count > 0 {
		return true, nil
	}
	return false, nil
}

//
func BuildingDel(ctx context.Context, req *pb_v1.BuildingDelRequest) (reply *pb_v1.BuildingDelReply, err error) {

	reply = &pb_v1.BuildingDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkBuildingDelParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	old := dbmodel.NewBuilding(dbname)
	where := map[string]interface{}{
		"id": req.Id,
		//"garden_id": req.GardenId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.BuildingUpdateRequest{
		Id:               req.Id,
		GardenId:         req.GardenId,
		BuildingName:     old.BuildingName,
		BuildingNumber:   old.BuildingNumber,
		BuildingArea:     old.BuildingArea,
		BuildingUsedArea: old.BuildingUsedArea,
	}

	house := dbmodel.NewHouse(dbname)
	where = map[string]interface{}{
		"building_id": req.Id,
		"status >":    HouseStatusWait,
	}
	count, err := house.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if count > 0 {
		return nil, status.Error(10003, "已入住房屋, 不能修改房屋编号，单元号，楼栋号")
	}

	houseIds, err := GetHouseIds(req.Id, 0, 0, dbname)
	if err != nil {
		return nil, err
	}

	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	// 删楼栋
	building := dbmodel.NewBuilding(dbname)
	where = map[string]interface{}{
		"id": req.Id,
	}
	err = building.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	// 删楼栋下的单元
	unit := dbmodel.NewUnit(dbname)
	where = map[string]interface{}{
		"building_id": req.Id,
	}
	err = unit.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	// 删楼栋下的房屋
	house = dbmodel.NewHouse(dbname)
	where = map[string]interface{}{
		"building_id": req.Id,
	}

	houseCount, err := house.Count(db, where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	err = house.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	isBind, err := IsHouseBindCharge(req.GardenId, houseIds, db)
	if err != nil {
		return nil, err
	}
	if isBind {
		return nil, status.Error(10003, "请先解除房屋绑定费用")
	}

	mg := dbmodel.NewBuildingManager(dbname)
	err = mg.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	mreq := pb_v1.ObjStatisticSetRequest{
		ObjType:       statistic.ObjTypeHouse,
		TotalIncrease: int64(0 - houseCount),
		GardenId:      req.GardenId,
	}
	err = statistic.ObjStatisticSetWithDb(&mreq, db)
	if err != nil {
		return nil, err
	}

	// 拒绝还未审批的房屋申请
	if err = RefuseHouseApply(req.GardenId, old.ID, 0, 0); err != nil {
		return nil, err
	}

	db.Commit()

	return reply, nil
}
