// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/charge_utils"
	dbmodel "property-garden/model"
	"property-garden/pb"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkHouseUpdateParam(req *pb_v1.HouseUpdateRequest) error {
	switch {
	case req.HouseNumber == "":
		return status.Error(10003, "门牌号不能为空")
	case strings.Contains(req.HouseNumber, "-"):
		return status.Error(10003, "门牌号不能包含-")
	case req.Layer < 1:
		return status.Error(10003, "楼层不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.RoomCount < 0 || req.HallCount < 0:
		return status.Error(10003, "几室几厅不能为空")
	case req.HouseType < 1:
		return status.Error(10003, "房屋类型不能为空")
	case req.UnitId < 1:
		return status.Error(10003, "单元id不能为空")
	case req.Id < 1:
		return status.Error(10003, "id不能为空")
	case req.HouseArea < 1:
		return status.Error(10003, "房屋面积不能为空")
	case req.HouseUsedArea < 1:
		return status.Error(10003, "房屋使用面积不能为空")
	}
	return nil
}

func updateChargeBindArea(houseId int64, area float64, chargeBasis int32, db *gorm.DB, dbname string) error {
	p := dbmodel.NewChargeBind(dbname)
	where := map[string]interface{}{
		"obj_id":       houseId,
		"obj_type":     charge_utils.ObjTypeHouse,
		"charge_basis": chargeBasis,
	}
	value := map[string]interface{}{
		"obj_area": area,
	}
	err := p.Update(db, where, value)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

func houseRentChangeField(req *pb_v1.HouseRentChangeFieldRequest, db *gorm.DB, dbname string) error {
	p := dbmodel.NewHouseRent(dbname)
	where := map[string]interface{}{}
	values := map[string]interface{}{}
	switch {
	case req.HouseId == 0 && req.UnitId == 0:
		if req.Lnt > 0 {
			values["lnt"] = req.Lnt
		}
		if req.Lat > 0 {
			values["lat"] = req.Lat
		}
		if req.GardenName != "" {
			values["garden_name"] = req.GardenName
		}
	case req.UnitId > 0:
		where["unit_id"] = req.UnitId
		hasLift := int32(1)
		if !req.HasLift {
			hasLift = 2
		}
		values["has_lift"] = hasLift
	case req.HouseId > 0:
		where["house_id"] = req.HouseId
		if req.Layer > 0 {
			values["layer"] = req.Layer
		}
		if req.HouseArea > 0 {
			values["house_area"] = req.HouseArea
		}
		if req.RoomCount > 0 {
			values["room_count"] = req.RoomCount
		}
		if req.HallCount > 0 {
			values["hall_count"] = req.HallCount
		}

	}
	if len(values) == 0 {
		return nil
	}

	err := p.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}

	_, err = pb.Household.HouseRentChangeField(context.Background(), req)
	return err
}

//
func HouseUpdate(ctx context.Context, req *pb_v1.HouseUpdateRequest) (reply *pb_v1.HouseUpdateReply, err error) {

	reply = &pb_v1.HouseUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkHouseUpdateParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	oldHouse := dbmodel.NewHouse(dbname)
	where := map[string]interface{}{
		"id": req.Id,
		//"garden_id":req.GardenId,

	}
	err = oldHouse.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if oldHouse.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.HouseUpdateRequest{
		Id:            req.Id,
		GardenId:      req.GardenId,
		UnitId:        oldHouse.UnitId,
		HouseNumber:   oldHouse.HouseNumber,
		RoomCount:     oldHouse.RoomCount,
		HallCount:     oldHouse.HallCount,
		Layer:         oldHouse.Layer,
		HouseType:     oldHouse.HouseType,
		HouseUsedArea: oldHouse.HouseUsedArea,
		HouseArea:     oldHouse.HouseArea,
	}

	newUnit := dbmodel.NewUnit(dbname)
	where = map[string]interface{}{
		"id": req.UnitId,
		//"garden_id":req.GardenId,
	}
	err = newUnit.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if newUnit.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	newBuilding := dbmodel.NewBuilding(dbname)
	where = map[string]interface{}{
		"id": newUnit.BuildingId,
		//"garden_id":req.GardenId,
	}
	err = newBuilding.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if newBuilding.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if oldHouse.Status > HouseStatusWait && (oldHouse.HouseNumber != req.HouseNumber ||
		newUnit.ID != oldHouse.UnitId) {
		return nil, status.Error(10003, "已入住房屋, 不能修改房屋编号，单元号，楼栋号")
	}

	now := time.Now()
	house := dbmodel.NewHouse(dbname)

	where = map[string]interface{}{
		"id": req.Id,
	}
	values := map[string]interface{}{
		"updated_at":      now,
		"has_lift":        newUnit.HasLift,
		"house_type":      req.HouseType,
		"house_number":    req.HouseNumber,
		"room_count":      req.RoomCount,
		"hall_count":      req.HallCount,
		"layer":           req.Layer,
		"unit_id":         req.UnitId,
		"building_id":     newUnit.BuildingId,
		"house_area":      req.HouseArea,
		"house_used_area": req.HouseUsedArea,
	}
	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	err = house.Update(db, where, values)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			return nil, errors.ErrRecordExist
		}
		return nil, errors.DataBaseError
	}
	if oldHouse.HouseUsedArea != req.HouseUsedArea {
		err = updateChargeBindArea(req.Id, req.HouseUsedArea, charge_utils.ChargeBasisUsedArea, db, dbname)
		if err != nil {
			return nil, err
		}
	}
	if oldHouse.HouseArea != req.HouseArea {
		err = updateChargeBindArea(req.Id, req.HouseArea, charge_utils.ChargeBasisArea, db, dbname)
		if err != nil {
			return nil, err
		}
	}

	if oldHouse.Status > HouseStatusWait && (oldHouse.HouseNumber != req.HouseNumber ||
		newUnit.ID != oldHouse.UnitId) {
		if err = RefuseHouseApply(req.GardenId, oldHouse.BuildingId, oldHouse.UnitId, oldHouse.ID); err != nil {
			return nil, err
		}
	}

	mreq := pb_v1.HouseRentChangeFieldRequest{
		GardenId:  req.GardenId,
		HouseId:   req.Id,
		RoomCount: req.RoomCount,
		Layer:     req.Layer,
		HouseArea: req.HouseArea,
		HallCount: req.HallCount,
	}

	err = houseRentChangeField(&mreq, db, dbname)
	if err != nil {
		return nil, err
	}

	db.Commit()

	return reply, nil
}
