// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	"property-garden/pb"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkBuildingManagerListParam(req *pb_v1.BuildingManagerListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.BuildingId < 1 && req.HouseId < 1:
		return status.Error(10003, "楼栋id和房屋id不能同时为空")

	}
	return nil

}

func getManagerInfos(ids []int64, gardenId int64) (map[int64]pb_v1.UserItem, error) {

	ret := map[int64]pb_v1.UserItem{}
	if len(ids) == 0 {
		return ret, nil
	}
	mreq := pb_v1.UserListRequest{
		GardenId: gardenId,
		Ids:      ids,
		PageSize: -1,
		Page:     -1,
	}
	mreply, err := pb.System.UserList(context.Background(), &mreq)
	if err != nil {
		return ret, err
	}
	for _, v := range mreply.List {
		ret[v.Id] = *v
	}

	return ret, nil

}

//
func BuildingManagerList(ctx context.Context, req *pb_v1.BuildingManagerListRequest) (reply *pb_v1.BuildingManagerListReply, err error) {

	reply = &pb_v1.BuildingManagerListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkBuildingManagerListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)

	if req.HouseId > 0 {
		house := dbmodel.NewHouse(dbname)
		where := map[string]interface{}{
			"id": req.HouseId,
		}
		err = house.Find(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound {
			return nil, errors.DataBaseError
		}
		if house.ID == 0 {
			return nil, errors.ErrRecordNotFound
		}
		req.BuildingId = house.BuildingId
	}

	p := dbmodel.NewBuildingManager(dbname)
	where := map[string]interface{}{
		"building_id": req.BuildingId,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)

	uids := []int64{}
	for _, v := range list {
		uids = append(uids, v.ManagerUid)
	}

	managerInfos, err := getManagerInfos(uids, req.GardenId)
	if err != nil {
		return nil, err
	}

	reply.List = make([]*pb_v1.BuildingManagerItem, len(list))

	for i, v := range list {
		item := &pb_v1.BuildingManagerItem{
			Id:         v.ID,
			ManagerUid: v.ManagerUid,
			Name:       managerInfos[v.ManagerUid].RealName,
			UserName:   managerInfos[v.ManagerUid].UserName,
			Phone:      managerInfos[v.ManagerUid].Phone,
			OpenimId:   managerInfos[v.ManagerUid].OpenimId,
		}
		reply.List[i] = item
	}

	return reply, nil
}
