// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/statistic"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkUnitDelParam(req *pb_v1.UnitDelRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")

	}
	return nil
}

//
func UnitDel(ctx context.Context, req *pb_v1.UnitDelRequest) (reply *pb_v1.UnitDelReply, err error) {

	reply = &pb_v1.UnitDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkUnitDelParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	oldUnit := dbmodel.NewUnit(dbname)
	where := map[string]interface{}{
		"id":        req.Id,
		//"garden_id": req.GardenId,
	}
	err = oldUnit.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if oldUnit.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.UnitUpdateRequest{
		Id:req.Id,
		GardenId:req.GardenId,
		UnitNumber:oldUnit.UnitNumber,
		UnitLayers:oldUnit.UnitLayers,
		UnitName:oldUnit.UnitName,
		BuildingId:oldUnit.BuildingId,
	}
	if oldUnit.HasLift == 1 {
		reply.Origin.HasLift = true
	}


	house := dbmodel.NewHouse(dbname)
	where = map[string]interface{}{
		"unit_id":   req.Id,
		"status >": HouseStatusWait,
	}
	count, err := house.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if count > 0 {
		return nil, status.Error(10003, "已入住房屋, 不能修改房屋编号，单元号，楼栋号")
	}

	houseIds, err := GetHouseIds(req.Id, 0, 0, dbname)
	if err != nil {
		return nil, err
	}

	db := database.DB().Begin()
	unit := dbmodel.NewUnit(dbname)
	where = map[string]interface{}{
		"id":req.Id,
	}
	err = unit.Delete(db, where)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	house = dbmodel.NewHouse(dbname)
	where = map[string]interface{}{
		"unit_id":req.Id,
	}
	houseCount, err := house.Count(database.DB(), where, nil)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	err = house.Delete(db, where)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	isBind, err := IsHouseBindCharge(req.GardenId, houseIds, db)
	if err != nil {
		db.Rollback()
		return nil, err
	}
	if isBind {
		db.Rollback()
		return nil, status.Error(10003, "请先解除房屋绑定费用")
	}
	mreq := pb_v1.ObjStatisticSetRequest{
		ObjType:statistic.ObjTypeHouse,
		TotalIncrease:int64(0-houseCount),
		GardenId:req.GardenId,
	}
	err = statistic.ObjStatisticSetWithDb(&mreq, db)
	if err != nil {
		db.Rollback()
		return nil, err
	}

	if err = RefuseHouseApply(req.GardenId, oldUnit.BuildingId, oldUnit.ID, 0); err != nil {
		db.Rollback()
		return nil, err
	}


	db.Commit()

	return reply, nil
}
