// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package household

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkGardenHouseholdUserListParam(req *pb_v1.GardenHouseholdUserListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "id不能为空")
	}

	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	return nil
}

//
func GardenHouseholdUserList(ctx context.Context, req *pb_v1.GardenHouseholdUserListRequest) (reply *pb_v1.GardenHouseholdUserListReply, err error) {

	reply = &pb_v1.GardenHouseholdUserListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenHouseholdUserListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)

	p := dbmodel.NewHouseApprovedGarden(dbname)
	where := map[string]interface{}{}
	if req.Name != "" {
		where["name like"] = "%" + req.Name + "%"
	}
	if req.IdNumber != "" {
		where["id_number"] = req.IdNumber
	}
	if req.Phone != "" {
		where["phone like"] = "%" + req.Phone + "%"
	}
	if req.UserType > 0 {
		where["user_type"] = req.UserType
	}
	if req.HouseId > 0 {
		where["house_id"] = req.HouseId
	}
	if len(req.Uids) > 0 {
		where["uid in"] = req.Uids
	}
	if req.BuildingId > 0 {
		where["building_id"] = req.BuildingId
	}
	if req.UnitId > 0 {
		where["unit_id"] = req.UnitId
	}
	if len(req.ExcludeUids) > 0 {
		where["uid not in ?"] = req.ExcludeUids
	}

	reply.Page = req.Page
	reply.Total, err = p.CountGroup(database.DB(), where, nil, "uid")
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.ListGroup(database.DB(), where, nil, "uid", int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}

	reply.List = make([]*pb_v1.GardenHouseholdUserItem, len(list))
	for i, v := range list {
		reply.List[i] = &pb_v1.GardenHouseholdUserItem{
			Id:        v.Uid,
			UserType:  v.UserType,
			IdType:    v.IdType,
			IdNumber:  v.IdNumber,
			Phone:     v.Phone,
			Name:      v.Name,
			HouseName: fmt.Sprintf("%s-%d-%s", v.BuildingNumber, v.UnitNumber, v.HouseNumber),
		}

		decrypt := utils.CertDecrypt(reply.List[i].IdNumber)
		if decrypt != "" {
			reply.List[i].IdNumber = decrypt
		}

	}

	return reply, nil

}
