// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package household

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
)

func checkGardenHouseholdListParam(req *pb_v1.GardenHouseholdListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "id不能为空")
	}

	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	return nil
}

//
func GardenHouseholdList(ctx context.Context, req *pb_v1.GardenHouseholdListRequest) (reply *pb_v1.GardenHouseholdListReply, err error) {

	reply = &pb_v1.GardenHouseholdListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenHouseholdListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	p := dbmodel.NewHouseApprovedGarden(dbname)

	where := map[string]interface{}{}
	if req.Uid > 0 {
		where["uid"] = req.Uid
	}
	if req.Name != "" {
		where["name"] = req.Name
	}
	if req.UserType > 0 {
		where["user_type"] = req.UserType
	}
	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}

	reply.List = make([]*pb_v1.GardenHouseholdItem, len(list))
	for i, v := range list {
		reply.List[i] = &pb_v1.GardenHouseholdItem{
			Id:            v.ID,
			HouseName:     fmt.Sprintf("%v-%v-%v", v.BuildingNumber, v.UnitNumber, v.HouseNumber),
			UserType:      v.UserType,
			ApproveStatus: 2,
			IdType:        v.IdType,
			IdNumber:      v.IdNumber,
			Phone:         v.Phone,
			Name:          v.Name,
			Uid:           v.Uid,
			HouseId:       v.HouseId,
			UnitId:        v.UnitId,
		}
		decrypt := utils.CertDecrypt(reply.List[i].IdNumber)
		if decrypt != "" {
			reply.List[i].IdNumber = decrypt
		}

		if v.Appendix != "" {
			reply.List[i].Appendix = strings.Split(v.Appendix, ";")
		}
		if !strings.Contains(v.HouseNumber, "-") {
			if v.UnitNumber > 0 {
				reply.List[i].HouseName = fmt.Sprintf("%s-%d-%s", v.BuildingNumber, v.UnitNumber, v.HouseNumber)
			} else {
				reply.List[i].HouseName = fmt.Sprintf("%s-%s", v.BuildingNumber, v.HouseNumber)
			}

		}
	}

	return reply, nil

}
