// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package household

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)

func checkGardenHouseholdUidsFromUnitIdParam(req *pb_v1.GardenHouseholdUidsFromUnitIdRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "id不能为空")
	case len(req.UnitIds) == 0:
		return status.Error(10003, "单元id不能为空")
	}

	return nil
}

type UidInfo struct {
	Uid int64 `gorm:"column:uid" json:"uid"`
}

//
func GardenHouseholdUidsFromUnitId(ctx context.Context, req *pb_v1.GardenHouseholdUidsFromUnitIdRequest) (reply *pb_v1.GardenHouseholdUidsFromUnitIdReply, err error) {

	reply = &pb_v1.GardenHouseholdUidsFromUnitIdReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenHouseholdUidsFromUnitIdParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	p := dbmodel.NewHouseApprovedGarden(dbname)

	where := map[string]interface{}{
		"unit_id in": req.UnitIds,
	}

	array := []UidInfo{}
	err = p.SelectGroup(database.DB(), where, nil, "uid", "uid", -1, -1, &array)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if len(array) == 0 {
		return reply, nil
	}
	reply.Uids = make([]int64, len(array))
	for i, v := range array {
		reply.Uids[i] = v.Uid
	}
	return reply, nil
}
