// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package neighbor

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
)


func checkNeighborArticleDelParam(req *pb_v1.NeighborArticleDelRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	case req.Uid == 0 && !req.Admin:
		return status.Error(10003, "uid 不能为空")
	}
	return nil

}


func NeighborArticleDel(ctx context.Context, req *pb_v1.NeighborArticleDelRequest) (reply *pb_v1.NeighborArticleDelReply, err error) {

	reply = &pb_v1.NeighborArticleDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkNeighborArticleDelParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	// 获取旧数据
	old := dbmodel.NewNeighborArticle(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if !req.Admin && old.Uid != req.Uid {
		return nil, status.Error(10003, "非本人不能删除")
	}

	class := dbmodel.NewNeighborClass(dbname)
	comment := dbmodel.NewNeighborComment(dbname)
	like := dbmodel.NewNeighborLike(dbname)

	db := database.DB().Begin()
	if err = old.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	where = map[string]interface{}{
		"article_id":req.Id,
	}

	if err = comment.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if err = like.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	where = map[string]interface{}{
		"id":old.ClassId,
	}
	values := map[string]interface{}{
		"article_count":gorm.Expr("article_count - ?", 1),
	}
	if err = class.Update(db, where, values); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if err := oss_utils.OssObjAdd(nil, strings.Split(old.Pics, ";")); err != nil {
		db.Rollback()
		return nil, err
	}


	db.Commit()

	return reply, nil
}
