// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package neighbor

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkNeighborArticleUpdateParam(req *pb_v1.NeighborArticleUpdateRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case len(req.Title) > 128:
		return status.Error(10003, "标题过长")
	case req.Title == "":
		return status.Error(10003, "标题不能为空")
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	case req.Uid == 0:
		return status.Error(10003, "uid 不能为空")
	}
	return nil
}

func NeighborArticleUpdate(ctx context.Context, req *pb_v1.NeighborArticleUpdateRequest) (reply *pb_v1.NeighborArticleUpdateReply, err error) {

	reply = &pb_v1.NeighborArticleUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkNeighborArticleUpdateParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewNeighborArticle(dbname)
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if old.Uid != req.Uid {
		return nil, status.Error(10003, "非本人不能修改")
	}

	// 更新新数据
	now := time.Now()
	article := dbmodel.NewNeighborArticle(dbname)
	values := map[string]interface{}{
		"title":      req.Title,
		"content":    req.Content,
		"updated_at": now,
		"pics":       utils.StringJoin(req.Pics, ";"),
	}

	where = map[string]interface{}{
		"id": req.Id,
	}
	db := database.DB().Begin()
	err = article.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if err := oss_utils.OssObjAdd(req.Pics, strings.Split(old.Pics, ";")); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	return reply, nil
}
