// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package neighbor

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)


func checkNeighborClassDelParam(req *pb_v1.NeighborClassDelRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	}
	return nil

}


func NeighborClassDel(ctx context.Context, req *pb_v1.NeighborClassDelRequest) (reply *pb_v1.NeighborClassDelReply, err error) {

	reply = &pb_v1.NeighborClassDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkNeighborClassDelParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	// 获取旧数据
	old := dbmodel.NewNeighborClass(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	article := dbmodel.NewNeighborArticle(dbname)
	comment := dbmodel.NewNeighborComment(dbname)
	like := dbmodel.NewNeighborLike(dbname)

	db := database.DB().Begin()
	if err = old.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	where["class_id"] = req.Id
	if err = article.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	if err = comment.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	if err = like.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if err := oss_utils.OssObjAdd(nil, []string{old.ClassPic}); err != nil {
		db.Rollback()
		return nil, err
	}

	db.Commit()

	return reply, nil
}
