// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package neighbor

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
)


func checkNeighborCommentDelParam(req *pb_v1.NeighborCommentDelRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	case req.Uid == 0 && !req.Admin:
		return status.Error(10003, "uid 不能为空")
	}
	return nil

}

func getSubCommentIds(id int64, db *gorm.DB, dbname string) ([]int64, error) {
	ret := []int64{id}
	ids := []int64{id}
	p := dbmodel.NewNeighborComment(dbname)
	for len(ids) > 0 {
		where := map[string]interface{}{
			"pid in":ids,
		}
		list, err := p.List(db, where, nil, -1, -1)
		if err != nil {
			return nil, errors.DataBaseError
		}
		ids = make([]int64, len(list))
		for i, v := range list {
			ids[i] = v.ID
		}
		ret = append(ret, ids...)
	}
	return ret, nil
}

func NeighborCommentDel(ctx context.Context, req *pb_v1.NeighborCommentDelRequest) (reply *pb_v1.NeighborCommentDelReply, err error) {

	reply = &pb_v1.NeighborCommentDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkNeighborCommentDelParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	// 获取旧数据
	old := dbmodel.NewNeighborComment(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if !req.Admin && old.Uid != req.Uid {
		return nil, status.Error(10003, "非本人不能删除")
	}

	article := dbmodel.NewNeighborArticle(dbname)
	comment := dbmodel.NewNeighborComment(dbname)
	like := dbmodel.NewNeighborLike(dbname)

	db := database.DB().Begin()

	// 删除本条评论
	if err = old.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	pids, err := getSubCommentIds(req.Id, db, dbname)
	if err != nil {
		db.Rollback()
		return nil, err
	}


	// 删除子评论
	where = map[string]interface{}{
		"pid in":pids,
	}

	if err = comment.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	// 删除评论相关点赞
	where = map[string]interface{}{
		"article_id":old.ArticleId,
		"like_type":2,
		"like_id in":pids,
	}
	if err = like.Delete(db, where); err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if old.Pid == 0 {

		// 减少文章评论数
		where = map[string]interface{}{
			"id":old.ArticleId,
		}
		values := map[string]interface{}{
			"comment_count":gorm.Expr("comment_count - ?", 1),
		}
		if err = article.Update(db, where, values); err != nil {
			db.Rollback()
			return nil, errors.DataBaseError
		}
	} else {
		// 减少父评论的评论数
		where = map[string]interface{}{
			"id":old.Pid,
		}
		values := map[string]interface{}{
			"sub_comment_count":gorm.Expr("sub_comment_count - ?", 1),
		}
		if err = comment.Update(db, where, values); err != nil {
			db.Rollback()
			return nil, errors.DataBaseError
		}
	}

	if err := oss_utils.OssObjAdd(nil, strings.Split(old.Pics, ";")); err != nil {
		db.Rollback()
		return nil, err
	}



	db.Commit()

	return reply, nil
}
