// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package neighbor

import (
	"context"
	"encoding/json"
	"fmt"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkNeighborCommentListParam(req *pb_v1.NeighborCommentListRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}

	return nil

}

func NeighborCommentList(ctx context.Context, req *pb_v1.NeighborCommentListRequest) (reply *pb_v1.NeighborCommentListReply, err error) {

	reply = &pb_v1.NeighborCommentListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkNeighborCommentListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	p := dbmodel.NewNeighborComment(dbname)
	where := map[string]interface{}{
		//"garden_id":req.GardenId,
	}
	if req.ArticleId > 0 {
		where["article_id"] = req.ArticleId
		where["pid"] = 0
	}
	if req.CommentId > 0 {
		where["pid"] = req.CommentId
	}

	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	likeIds := make([]int64, len(list))
	uids := []int64{}
	for i, v := range list {
		likeIds[i] = v.ID
		uids = append(uids, v.Uid)
	}
	userInfoM, err := getUserInfo(uids, dbname)
	likeInfoM, err := getLikeInfo(req.Uid, likeIds, dbname, 2)
	if err != nil {
		return nil, err
	}
	reply.List = make([]*pb_v1.NeighborCommentItem, len(list))
	for i, v := range list {

		item := &pb_v1.NeighborCommentItem{
			Content:         v.Content,
			Pics:            strings.Split(v.Pics, ";"),
			Id:              v.ID,
			CreatedAt:       v.CreatedAt.Format("2006-01-02 15:04:05"),
			UpdatedAt:       v.UpdatedAt.Format("2006-01-02 15:04:05"),
			Likes:           v.Likes,
			Like:            likeInfoM[v.ID],
			Uid:             v.Uid,
			UserIcon:        v.UserIcon,
			UserName:        userInfoM[v.Uid].RealName,
			UserPhone:       userInfoM[v.Uid].Phone,
			SubCommentCount: v.SubCommentCount,
			Owner:           req.Uid == v.Uid,
			NickName:        userInfoM[v.Uid].NickName,
		}

		reply.List[i] = item
	}

	return reply, nil
}
