// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package neighbor

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)



func checkNeighborLikeParam(req *pb_v1.NeighborLikeRequest) error {

	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.LikeType == 0:
		return status.Error(10003, "点赞目标类型不能为空")
	case req.LikeId == 0:
		return status.Error(10003, "点赞目标id不能为空")
	case req.Uid == 0:
		return status.Error(10003, "uid 不能为空")

	}
	return nil
}

func addArticleLike(req *pb_v1.NeighborLikeRequest, db *gorm.DB, dbname string) ( err error) {
	article := dbmodel.NewNeighborArticle(dbname)
	where := map[string]interface{}{
		"id":req.LikeId,
	}
	err = article.Find(db, where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return  errors.DataBaseError
	}
	if article.ID == 0 {
		return errors.ErrRecordNotFound
	}
	now := time.Now()

	p := dbmodel.TNeighborLike{
		ClassId:article.ClassId,
		ArticleId:article.ID,
		LikeId:req.LikeId,
		CreatedAt:now,
		UpdatedAt:now,
		LikeType:req.LikeType,
		Uid:req.Uid,
	}
	p.SetTable(dbname)
	err = p.Insert(db)
	if err != nil {
		return errors.DataBaseError
	}
	values := map[string]interface{}{
		"likes":gorm.Expr("likes + ?", 1),
	}
	err = article.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

func delArticleLike(req *pb_v1.NeighborLikeRequest, db *gorm.DB, dbname string) (err error) {
	article := dbmodel.NewNeighborArticle(dbname)
	where := map[string]interface{}{
		"id":req.LikeId,
	}
	err = article.Find(db, where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return  errors.DataBaseError
	}
	if article.ID == 0 {
		return errors.ErrRecordNotFound
	}

	p := dbmodel.NewNeighborLike(dbname)
	where = map[string]interface{}{
		"uid":req.Uid,
		"like_type":req.LikeType,
		"like_id":req.LikeId,
	}
	err = p.Delete(db, where)
	if err != nil {
		return errors.DataBaseError
	}
	where = map[string]interface{}{
		"id":req.LikeId,
	}
	values := map[string]interface{}{
		"likes":gorm.Expr("likes - ?", 1),
	}
	err = article.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

func addCommentLike(req *pb_v1.NeighborLikeRequest, db *gorm.DB, dbname string) (err error) {
	comment := dbmodel.NewNeighborComment(dbname)
	where := map[string]interface{}{
		"id":req.LikeId,
	}
	err = comment.Find(db, where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return  errors.DataBaseError
	}
	if comment.ID == 0 {
		return errors.ErrRecordNotFound
	}
	now := time.Now()

	p := dbmodel.TNeighborLike{
		ClassId:comment.ClassId,
		ArticleId:comment.ArticleId,
		LikeId:req.LikeId,
		CreatedAt:now,
		UpdatedAt:now,
		LikeType:req.LikeType,
		Uid:req.Uid,
	}
	p.SetTable(dbname)
	err = p.Insert(db)
	if err != nil {
		return errors.DataBaseError
	}
	values := map[string]interface{}{
		"likes":gorm.Expr("likes + ?", 1),
	}
	err = comment.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}
	return nil

}

func delCommentLike(req *pb_v1.NeighborLikeRequest, db *gorm.DB, dbname string) (err error) {
	comment := dbmodel.NewNeighborComment(dbname)
	where := map[string]interface{}{
		"id":req.LikeId,
	}
	err = comment.Find(db, where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return  errors.DataBaseError
	}
	if comment.ID == 0 {
		return errors.ErrRecordNotFound
	}

	p := dbmodel.NewNeighborLike(dbname)
	where = map[string]interface{}{
		"uid":req.Uid,
		"like_type":req.LikeType,
		"like_id":req.LikeId,
	}
	err = p.Delete(db, where)
	if err != nil {
		return errors.DataBaseError
	}
	where = map[string]interface{}{
		"id":req.LikeId,
	}
	values := map[string]interface{}{
		"likes":gorm.Expr("likes - ?", 1),
	}
	err = comment.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

func NeighborLike(ctx context.Context, req *pb_v1.NeighborLikeRequest) (reply *pb_v1.NeighborLikeReply, err error) {

	reply = &pb_v1.NeighborLikeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkNeighborLikeParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	old := dbmodel.NewNeighborLike(dbname)
	where := map[string]interface{}{
		"uid":req.Uid,
		"like_type":req.LikeType,
		"like_id":req.LikeId,
	}

	count, err := old.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if count == 0 && !req.Like{
		return nil, status.Error(10003, "未点赞")
	}
	if count > 0 && req.Like {
		return nil, status.Error(10003, "已点赞")
	}

	db := database.DB().Begin()
	switch  {
	case req.Like && req.LikeType == 1:
		err = addArticleLike(req, db, dbname)
	case req.Like && req.LikeType == 2:
		err = addCommentLike(req, db, dbname)
	case !req.Like && req.LikeType == 1:
		err = delArticleLike(req, db, dbname)
	case !req.Like && req.LikeType == 2:
		err = delCommentLike(req, db, dbname)

	}
	if err != nil {
		db.Rollback()
		return nil, err
	}

	db.Commit()

	return reply, nil

}
