// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package park

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"
)


func checkParkUpdateParam(req *pb_v1.ParkUpdateRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.ParkName == "":
		return status.Error(10003, "停车场名称不能为空")
	case req.ParkNumber == "":
		return status.Error(10003, "停车场编号不能为空")
	case req.ParkType < 1:
		return status.Error(10003, "停车场类型不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	}
	return nil

}

//
func ParkUpdate(ctx context.Context, req *pb_v1.ParkUpdateRequest) (reply *pb_v1.ParkUpdateReply, err error) {

	reply = &pb_v1.ParkUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkParkUpdateParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	now := time.Now()
	p := dbmodel.NewPark(dbname)
	where := map[string]interface{}{
		"id":req.Id,
		//"garden_id":req.GardenId,
	}

	values := map[string]interface{}{
		"park_name":req.ParkName,
		"park_number":req.ParkNumber,
		"park_type":req.ParkType,
		"comment":req.Comment,
		"updated_at":now,
	}

	old := dbmodel.NewPark(dbname)
	err = old.Find(database.DB(), where)

	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if req.ParkNumber != old.ParkNumber {
		// todo
		//isBind, err := isParkBindCharge(req.Id, dbname)
		//if err != nil {
		//	return nil, err
		//}
		//if isBind {
		//	return nil, status.Error(10003, "该区域有车位已绑定费用，请先解除绑定")
		//}
	}


	reply.Origin = &pb_v1.ParkUpdateRequest{
		Id:req.Id,
		ParkType:old.ParkType,
		ParkNumber:old.ParkNumber,
		ParkName:old.ParkName,
		Comment:old.Comment,
		GardenId:req.GardenId,
	}

	err = p.Update(database.DB(), where, values)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			return nil, status.Error(10003, "该停车场已存在")
		}

		return nil, errors.DataBaseError
	}

	return reply, nil
}
