// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package property_package

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

const (
	PropertyPackageTypeMonth = 1
	PropertyPackageTypeText = 2
)
var propertyPackageTypeM = map[int32]string {
	PropertyPackageTypeMonth:"按月",
	PropertyPackageTypeText:"按文本",
}

func checkPropertyPackageAddParam(req *pb_v1.PropertyPackageAddRequest) error {

	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case propertyPackageTypeM[req.PackageType] == "":
		return status.Error(10003, "赠送类型错误")
	case req.PackageType == PropertyPackageTypeMonth && req.GiveMonths < 1:
		return status.Error(10003, "赠送月数不能为空")
	case req.PackageType == PropertyPackageTypeText && req.GiveContent == "":
		return status.Error(10003, "赠送文本不能为空")
	case req.PayMonths < 1:
		return status.Error(10003, "缴费月数不能为空")
	}
	return nil
}


func PropertyPackageAdd(ctx context.Context, req *pb_v1.PropertyPackageAddRequest) (reply *pb_v1.PropertyPackageAddReply, err error) {

	reply = &pb_v1.PropertyPackageAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkPropertyPackageAddParam(req)
	if err != nil {
		return nil, err
	}
	now := time.Now()
	dbname := utils.GetGardenDbName(req.GardenId)

	if req.PackageType == PropertyPackageTypeMonth {
		p := dbmodel.NewPropertyPackage(dbname)
		where := map[string]interface{}{
			"package_type":PropertyPackageTypeMonth,
			"give_months":req.GiveMonths,
			"pay_months":req.PayMonths,
		}
		err = p.Find(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound{
			return nil, errors.DataBaseError
		}
		if p.ID > 0 {
			if p.Enable == 2 {
				return nil, status.Error(10003, "相同套餐已存在, 请启用")
			}
			return nil, status.Error(10003, "相同套餐已存在")
		}
	}

	enable := int32(1)
	if !req.Enable {
		enable = 2
	}
	p := dbmodel.TPropertyPackage{
		GiveContent:req.GiveContent,
		GiveMonths:req.GiveMonths,
		PackageType:req.PackageType,
		PayMonths:req.PayMonths,
		Enable:enable,
		UpdatedAt:now,
		CreatedAt:now,
	}
	p.SetTable(dbname)
	err = p.Insert(database.DB())
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.Id = p.ID

	return reply, nil
}
