// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package property_package

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkPropertyPackageUpdateParam(req *pb_v1.PropertyPackageUpdateRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case propertyPackageTypeM[req.PackageType] == "":
		return status.Error(10003, "赠送类型错误")
	case req.PackageType == PropertyPackageTypeMonth && req.GiveMonths < 1:
		return status.Error(10003, "赠送月数不能为空")
	case req.PackageType == PropertyPackageTypeText && req.GiveContent == "":
		return status.Error(10003, "赠送文本不能为空")
	case req.PayMonths < 1:
		return status.Error(10003, "缴费月数不能为空")
	case req.Id < 1:
		return status.Error(10003, "id 不能为空")
	}
	return nil
}




func PropertyPackageUpdate(ctx context.Context, req *pb_v1.PropertyPackageUpdateRequest) (reply *pb_v1.PropertyPackageUpdateReply, err error) {

	reply = &pb_v1.PropertyPackageUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkPropertyPackageUpdateParam(req)
	if err != nil {
		return nil, err
	}


	dbname := utils.GetGardenDbName(req.GardenId)

	if req.PackageType == PropertyPackageTypeMonth {
		p := dbmodel.NewPropertyPackage(dbname)
		where := map[string]interface{}{
			"package_type":PropertyPackageTypeMonth,
			"give_months":req.GiveMonths,
			"pay_months":req.PayMonths,
			"id !=":req.Id,
		}
		err = p.Find(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound{
			return nil, errors.DataBaseError
		}
		if p.ID > 0 {
			if p.Enable == 2 {
				return nil, status.Error(10003, "已存在相同套餐, 请启用")
			}
			return nil, status.Error(10003, "已存在相同套餐")
		}
	}


	// 获取旧数据
	old := dbmodel.NewPropertyPackage(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.PropertyPackageUpdateRequest{
		GardenId:req.GardenId,
		Id:req.Id,
		GiveMonths:old.GiveMonths,
		GiveContent:old.GiveContent,
		PayMonths:old.PayMonths,
		PackageType:old.PackageType,
		Enable:false,
	}
	if old.Enable == 1 {
		reply.Origin.Enable = true
	}

	// 更新新数据
	now := time.Now()
	propertyPackage := dbmodel.NewPropertyPackage(dbname)
	enable := 1
	if !req.Enable {
		enable = 2
	}
	values := map[string]interface{}{
		"give_months":req.GiveMonths,
		"give_content":req.GiveContent,
		"package_type":req.PackageType,
		"enable":enable,
		"pay_months":req.PayMonths,
		"updated_at":now,

	}

	where = map[string]interface{}{
		"id":req.Id,
	}
	err = propertyPackage.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
