// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package repair

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkRepairClassUpdateParam(req *pb_v1.RepairClassUpdateRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Name ==  "":
		return status.Error(10003, "名称不能为空")
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	}
	return nil
}




func RepairClassUpdate(ctx context.Context, req *pb_v1.RepairClassUpdateRequest) (reply *pb_v1.RepairClassUpdateReply, err error) {

	reply = &pb_v1.RepairClassUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkRepairClassUpdateParam(req)
	if err != nil {
		return nil, err
	}


	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewRepairClass(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.RepairClassUpdateRequest{
		GardenId:req.GardenId,
		Id:req.Id,
		Name:req.Name,
		Enable:false,
	}
	if old.Enable == 1 {
		reply.Origin.Enable = true
	}

	if old.Enable == 1 && !req.Enable {
		repairOrder := dbmodel.NewRepairOrder(dbname)
		where := map[string]interface{}{
			"status in":[]int32{RepairOrderStatusSend, RepairOrderStatusCreate},
			"class_id":req.Id,
		}
		count, err := repairOrder.Count(database.DB(), where, nil)
		if err != nil {
			return nil, errors.DataBaseError
		}
		if count > 0 {
			return nil, status.Error(10003, "有工单未处理，不能禁用")
		}
	}



	// 更新新数据
	now := time.Now()
	class := dbmodel.NewRepairClass(dbname)
	enable := 1
	if !req.Enable {
		enable = 2
	}
	values := map[string]interface{}{
		"name":req.Name,
		"enable":enable,
		"updated_at":now,
		"area":req.Area,

	}

	where = map[string]interface{}{
		"id":req.Id,
	}
	err = class.Update(database.DB(), where, values)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			return nil, status.Error(10003, "分类名已存在")
		}
		return nil, errors.DataBaseError
	}

	return reply, nil
}
