// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package repair

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/statistic"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkRepairOrderDelParam(req *pb_v1.RepairOrderDelRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	return nil
}




func RepairOrderDel(ctx context.Context, req *pb_v1.RepairOrderDelRequest) (reply *pb_v1.RepairOrderDelReply, err error) {

	reply = &pb_v1.RepairOrderDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkRepairOrderDelParam(req)
	if err != nil {
		return nil, err
	}


	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewRepairOrder(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if old.Status == RepairOrderStatusSend {
		return nil, status.Error(10003, "已派单，不能删除")
	}
	if req.HouseholdUid > 0  && old.HouseholdUid != req.HouseholdUid{
		return nil, status.Error(10003,"您无法进行该操作")
	}

	reply.Origin = &pb_v1.RepairOrderUpdateRequest{
		GardenId:req.GardenId,
		Id:req.Id,

		// 分类id
		ClassId:old.ClassId,
		// 报修人
		ApplyPeople:old.ApplyPeople,
		// 报修人电话
		ApplyPeoplePhone:old.ApplyPeoplePhone,
		// 预约时间
		Appointment: old.Appointment.Unix(),
		// 房屋id
		HouseId:old.HouseId,
		HouseholdUid:old.HouseholdUid,
		// 报修内容
		ApplyContent:old.ApplyContent,
		// 报修图片
		ApplyPic:strings.Split(old.ApplyPic, ";"),

	}


	// 删除数据
	repairOrder := dbmodel.NewRepairOrder(dbname)

	where = map[string]interface{}{
		"id":req.Id,
	}
	db := database.DB().Begin()
	err = repairOrder.Delete(db, where)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	statisticReq := pb_v1.RepairStatisticSetRequest{
		HandleType:statistic.HandleTypeRepair,
		TotalIncrease:int64(-1),
		GardenId:req.GardenId,
	}
	if old.Status == RepairOrderStatusFinish {
		statisticReq.FinishIncrease = -1
	}

	err = statistic.RepairStatisticSetWithDb(&statisticReq, db)

	if err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	return reply, nil
}
