// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package repair

import (
	"context"
	"encoding/json"
	"fmt"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)


func checkRepairOrderListParam(req *pb_v1.RepairOrderListRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}

	return nil

}

func getClassName(ids []int64, dbname string) (map[int64]dbmodel.TRepairClass, error) {
	ret := map[int64]dbmodel.TRepairClass{}
	if len(ids) == 0 {
		return ret, nil
	}
	p := dbmodel.NewRepairClass(dbname)
	where := map[string]interface{}{
		"id in":ids,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range list {
		ret[v.ID] = v
	}

	return ret, nil
}


func RepairOrderList(ctx context.Context, req *pb_v1.RepairOrderListRequest) (reply *pb_v1.RepairOrderListReply, err error) {

	reply = &pb_v1.RepairOrderListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkRepairOrderListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	p := dbmodel.NewRepairOrder(dbname)
	where := map[string]interface{}{}
	if req.CurrentUid > 0 {
		where["current_uid"] = req.CurrentUid
	}
	if req.Status > 0 {
		where["status"] = req.Status
	}
	if req.ApplyPeople != "" {
		where["apply_people"] = req.ApplyPeople
	}
	if req.ApplyPeoplePhone != "" {
		where["apply_people_phone"] = req.ApplyPeoplePhone
	}
	if req.ClassId > 0 {
		where["class_id"] = req.ClassId
	}
	if req.HouseholdUid > 0 {
		where["household_uid"] = req.HouseholdUid
	}

	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.RepairOrderItem, len(list))

	uids := []int64{}
	classIds := []int64{}
	for _, v := range list {
		classIds = append(classIds, v.ClassId)
		if v.CurrentUid > 0 {
			uids = append(uids, v.CurrentUid)
		}
		if !v.LastUidIsCompany && v.LastUid > 0 {
			uids = append(uids, v.LastUid)
		}

	}
	userInfos, err := getUserInfos(uids, req.GardenId)
	if err != nil {
		return nil, err
	}
	classInfos, err := getClassName(classIds, dbname)
	if err != nil {
		return nil, err
	}

	for i, v := range list {

		item := &pb_v1.RepairOrderItem{
			// 分类id
			ClassId:v.ClassId,
			// 报修人
			ApplyPeople:v.ApplyPeople,
			// 报修人电话
			ApplyPeoplePhone:v.ApplyPeoplePhone,
			// 预约时间
			Appointment:v.Appointment.Format("2006-01-02 15:04:05"),
			// 房屋id
			HouseId:v.HouseId,
			// 处理人
			CurrentUser:userInfos[v.CurrentUid].Name,
			// 报修内容
			ApplyContent:v.ApplyContent,
			// 报修图片
			ApplyPic:strings.Split(v.ApplyPic, ";"),
			// 回访内容
			ReturnVisitContent: v.ReturnVisitContent,
			// 回访满意度
			ReturnVisitLevel: v.ReturnVisitLevel,
			// 楼栋id
			BuildingId:v.BuildingId,
			// 单元id
			UnitId:v.UnitId,
			// 房屋全称
			HouseName:v.HouseName,
			// 报修时间
			CreatedAt:v.CreatedAt.Format("2006-01-02 15:04:05"),
			// 维修时间
			RepairTime:"",
			// 分类名称
			ClassName: classInfos[v.ClassId].Name,
			Id:v.ID,
			Status:v.Status,
			Area:classInfos[v.ClassId].Area,
		}
		if !v.LastUidIsCompany {
			item.LastUser = userInfos[v.LastUid].Name
		}
		if v.Status == RepairOrderStatusFinish {
			item.RepairTime = v.RepairTime.Format("2006-01-02 15:04:05")
		}
		if v.Appointment.Unix() < 1136131200 {
			item.Appointment = ""
		}
		reply.List[i] = item
	}

	return reply, nil
}
