// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package repair

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/public_msg"
	"property-garden/impl/v1/system_msg"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkRepairOrderSendParam(req *pb_v1.RepairOrderSendRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.CurrentUid < 1:
		return status.Error(10003, "当前操作人不能为空")
	case req.LastUid < 1:
		return status.Error(10003, "上级操作人不能为空")
	}
	return nil
}

func RepairOrderSend(ctx context.Context, req *pb_v1.RepairOrderSendRequest) (reply *pb_v1.RepairOrderSendReply, err error) {

	reply = &pb_v1.RepairOrderSendReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkRepairOrderSendParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewRepairOrder(dbname)
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if old.Status == RepairOrderStatusFinish {
		return nil, status.Error(10003, "工单已完结，不支持该操作")
	}

	// 工单流水线
	pipelineArray := []RepairOrderPipelineData{}
	json.Unmarshal([]byte(old.Pipeline), &pipelineArray)
	now := time.Now()
	pipeline := RepairOrderPipelineData{LastUid: req.LastUid,
		CurrentUid: req.CurrentUid, Status: RepairPipelineStatusSend,
		LastByCompany: req.ByCompany,
		Feedback:      req.Feedback, HandleTime: now.Format("2006-01-02 15:04:05")}
	if old.Status == RepairOrderStatusSend {
		pipeline.Status = RepairPipelineStatusResend
		if req.Feedback == "" {
			return nil, status.Error(10003, "转单说明不能为空")
		}
	}
	pipelineArray = append(pipelineArray, pipeline)
	bytes, _ := json.Marshal(pipelineArray)

	// 更新新数据
	repairOrder := dbmodel.NewRepairOrder(dbname)

	values := map[string]interface{}{
		"status":              RepairOrderStatusSend,
		"current_uid":         req.CurrentUid,
		"last_uid":            req.LastUid,
		"feedback":            req.Feedback,
		"updated_at":          now,
		"pipeline":            string(bytes),
		"last_uid_is_company": req.ByCompany,
	}

	where = map[string]interface{}{
		"id": req.Id,
	}
	err = repairOrder.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	if old.HouseholdUid > 0 {
		serviceInfo := public_msg.ServiceInfo{
			Name:  "报事报修",
			State: "已派单",
		}

		public_msg.SendMsgWx(old.HouseholdUid, req.GardenId, serviceInfo)
	}

	mreq := pb_v1.SystemMsgAddRequest{
		GardenId: req.GardenId,
		Content:  "报事报修派单",
		Code:     system_msg.SystemMsgCodeRepairOrderSended,
		Uid:      req.CurrentUid,
	}
	system_msg.SystemMsgAdd(ctx, &mreq)

	return reply, nil
}
