// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package repair

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkRepairOrderUpdateParam(req *pb_v1.RepairOrderUpdateRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.ClassId < 1:
		return status.Error(10003, "报修类型不能为空")
	case req.ApplyPeople == "":
		return status.Error(10003, "报修人不能为空")
	case req.ApplyPeoplePhone != "":
		if !utils.VerifyMobileFormat(req.ApplyPeoplePhone) {
			return status.Error(10003, "手机格式错误")
		}
	case req.ApplyContent == "":
		return status.Error(10003, "报修内容不能为空")
	}
	return nil
}

func RepairOrderUpdate(ctx context.Context, req *pb_v1.RepairOrderUpdateRequest) (reply *pb_v1.RepairOrderUpdateReply, err error) {

	reply = &pb_v1.RepairOrderUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkRepairOrderUpdateParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewRepairOrder(dbname)
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if req.HouseholdUid > 0 && old.HouseholdUid != req.HouseholdUid {
		return nil, status.Error(10003, "您无法进行该操作")
	}

	reply.Origin = &pb_v1.RepairOrderUpdateRequest{
		GardenId: req.GardenId,
		Id:       req.Id,

		// 分类id
		ClassId: old.ClassId,
		// 报修人
		ApplyPeople: old.ApplyPeople,
		// 报修人电话
		ApplyPeoplePhone: old.ApplyPeoplePhone,
		// 预约时间
		Appointment: old.Appointment.Unix(),
		// 房屋id
		HouseId: old.HouseId,
		// 报修内容
		ApplyContent: old.ApplyContent,
		HouseholdUid: old.HouseholdUid,
		// 报修图片
		ApplyPic: strings.Split(old.ApplyPic, ";"),
	}

	// 更新新数据
	now := time.Now()
	repairOrder := dbmodel.NewRepairOrder(dbname)

	values := map[string]interface{}{
		"class_id": req.ClassId,
		// 报修人
		"apply_people": req.ApplyPeople,
		// 报修人电话
		"apply_people_phone": req.ApplyPeoplePhone,
		// 预约时间
		"appointment": time.Unix(req.Appointment, 0),
		// 报修内容
		"apply_content": req.ApplyContent,
		// 报修图片
		"apply_pic":  utils.StringJoin(req.ApplyPic, ";"),
		"updated_at": now,
	}
	if req.HouseId == 0 {

		values["building_id"] = 0
		values["unit_id"] = 0
		values["house_id"] = 0
		values["house_name"] = ""

	}

	if req.HouseId > 0 && req.HouseId != old.HouseId {
		houseInfo, err := getHouseInfo(req.HouseId, req.GardenId)
		if err != nil {
			return nil, errors.DataBaseError
		}
		values["building_id"] = houseInfo.BuildingId
		values["unit_id"] = houseInfo.UnitId
		values["house_id"] = req.HouseId
		values["house_name"] = houseInfo.HouseName
	}

	where = map[string]interface{}{
		"id": req.Id,
	}
	db := database.DB().Begin()
	err = repairOrder.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	if err := oss_utils.OssObjAdd(req.ApplyPic, strings.Split(old.ApplyPic, ";")); err != nil {
		db.Rollback()
		return nil, err
	}

	db.Commit()

	return reply, nil
}
