// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package statistic

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
)




func checkStatisticParam(req *pb_v1.StatisticRequest) error {

	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	return nil
}


func Statistic(ctx context.Context, req *pb_v1.StatisticRequest) (reply *pb_v1.StatisticReply, err error) {

	reply = &pb_v1.StatisticReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkStatisticParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)

	obj := dbmodel.NewStatisticObj(dbname)
	objList, err := obj.List(database.DB(), nil, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	repair := dbmodel.NewStatisticRepair(dbname)
	repairList, err := repair.List(database.DB(), nil, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range objList {
		switch v.ObjType {
		case ObjTypeHouse:
			reply.HouseCount = v.Total
		case ObjTypeSpace:
			reply.SpaceCount = v.Total
		case ObjTypeVehicle:
			reply.VehicleCount = v.Total
		case ObjTypeUser:
			reply.UserCount = v.Total
		}
	}
	reply.List = []*pb_v1.StatisticWaitFinish{}
	for _, v := range repairList {
		item := &pb_v1.StatisticWaitFinish{
			Type:StatisticDesc[int(v.HandleType)],
			WaitCount:v.Total - v.Finish,
			FinishCount:v.Finish,
		}
		if item.WaitCount < 0 {
			item.WaitCount = 0
		}
		if item.FinishCount < 0 {
			item.FinishCount = 0
		}
		reply.List = append(reply.List, item)

	}


	return reply, err
}
