// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package suggestion

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkSuggestionOrderBackParam(req *pb_v1.SuggestionOrderBackRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.LastUid < 1:
		return status.Error(10003, "当前操作人不能为空")
	//case req.NextUid < 1:
	//	return status.Error(10003, "下级操作人不能为空")
	case req.Feedback == "":
		return status.Error(10003, "退单理由不能为空")
	}
	return nil
}




func SuggestionOrderBack(ctx context.Context, req *pb_v1.SuggestionOrderBackRequest) (reply *pb_v1.SuggestionOrderBackReply, err error) {

	reply = &pb_v1.SuggestionOrderBackReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkSuggestionOrderBackParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewSuggestionOrder(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if old.Status != SuggestionOrderStatusSend {
		return nil, status.Error(10003, "当前状态不支持该操作")
	}

	// 工单流水线
	pipelineArray := []SuggestionOrderPipelineData{}
	json.Unmarshal([]byte(old.Pipeline), &pipelineArray)
	now := time.Now()
	pipeline := SuggestionOrderPipelineData{LastUid:req.LastUid,
		CurrentUid:0, Status:SuggestionPipelineStatusBack,
		LastByCompany:req.ByCompany,
		Feedback:req.Feedback, HandleTime:now.Format("2006-01-02 15:04:05")}

	pipelineArray = append(pipelineArray, pipeline)
	bytes, _ := json.Marshal(pipelineArray)

	// 更新新数据
	repairOrder := dbmodel.NewSuggestionOrder(dbname)

	values := map[string]interface{}{
		"status":SuggestionOrderStatusCreate,
		"last_uid":req.LastUid,
		"current_uid":0,
		"feedback": req.Feedback,
		"updated_at":now,
		"pipeline":string(bytes),
		"last_uid_is_company":req.ByCompany,
	}

	where = map[string]interface{}{
		"id":req.Id,
	}
	err = repairOrder.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
