// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package suggestion

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	dbmodel "property-garden/model"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkSuggestionOrderReturnVisitParam(req *pb_v1.SuggestionOrderReturnVisitRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "id 不能为空")
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.ReturnVisitLevel < 1:
		return status.Error(10003, "满意度不能为空")
	}
	return nil
}

func SuggestionOrderReturnVisit(ctx context.Context, req *pb_v1.SuggestionOrderReturnVisitRequest) (reply *pb_v1.SuggestionOrderReturnVisitReply, err error) {

	reply = &pb_v1.SuggestionOrderReturnVisitReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkSuggestionOrderReturnVisitParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewSuggestionOrder(dbname)
	where := map[string]interface{}{
		"id": req.Id,
	}
	if req.HouseholdUid > 0 {
		where["household_uid"] = req.HouseholdUid
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if old.Status != SuggestionOrderStatusFinish {
		return nil, status.Error(10003, "工单未完结，不支持该操作")
	}

	// 工单流水线
	pipelineArray := []pb_v1.SuggestionOrderPipelineData{}
	json.Unmarshal([]byte(old.Pipeline), &pipelineArray)
	now := time.Now()

	// 更新新数据
	repairOrder := dbmodel.NewSuggestionOrder(dbname)

	values := map[string]interface{}{

		"return_visit_level":   req.ReturnVisitLevel,
		"return_visit_content": req.ReturnVisitContent,
		"updated_at":           now,
	}
	where = map[string]interface{}{
		"id": req.Id,
	}
	if req.HouseholdUid > 0 {
		where["household_uid"] = req.HouseholdUid
	}
	err = repairOrder.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
