// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package vote

import (
	"context"
	"encoding/json"
	"fmt"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	"property-garden/parser"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkVoteAddParam(req *pb_v1.VoteAddRequest) error {

	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Title == "":
		return status.Error(10003, "标题不能为空")
	case req.Start < 1 || req.End < 1:
		return status.Error(10003, "开始和结束时间不能为空")
	case req.Start >= req.End:
		return status.Error(10003, "开始时间不能晚于结束时间")
	case req.End <= time.Now().Unix():
		return status.Error(10003, "结束时间不能早于当前时间")
	case len(req.Topics) == 0:
		return status.Error(10003, "题目不能为空")
	}
	pic := []string{}
	for _, v := range req.Pics {
		if oss_utils.IsFix(v) {
			continue
		}
		pic = append(pic, v)
	}
	req.Pics = pic

	flagM := map[string]bool{}
	for _, v := range req.Topics {
		if v.TopicType != TopicTypeMultiChoice && v.TopicType != TopicTypeSingleChoice {
			continue
		}
		for _, c := range v.ChoiceItems {
			if c.Flag == "" || c.Text == "" {
				return status.Error(10003, "选择题的选项和标签不能为空")
			}
			if flagM[c.Flag] {
				return status.Error(10003, "选择题的标签有重复")
			}
			flagM[c.Flag] = true
		}
		flagM = map[string]bool{}

	}
	return nil
}

func VoteAdd(ctx context.Context, req *pb_v1.VoteAddRequest) (reply *pb_v1.VoteAddReply, err error) {

	reply = &pb_v1.VoteAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkVoteAddParam(req)
	if err != nil {
		return nil, err
	}
	for i, _ := range req.Topics {
		req.Topics[i].Number = int64(i + 1)
	}
	now := time.Now()
	dbname := utils.GetGardenDbName(req.GardenId)
	topics, _ := json.Marshal(req.Topics)

	p := dbmodel.TVote{
		Start:     req.Start,
		End:       req.End,
		Title:     req.Title,
		Topics:    string(topics),
		UpdatedAt: now,
		CreatedAt: now,
	}
	if len(req.Pics) > 0 {
		p.Pics = utils.StringJoin(req.Pics, ";")
	}
	p.SetTable(dbname)
	db := database.DB().Begin()
	err = p.Insert(db)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	if err := oss_utils.OssObjAdd(req.Pics, nil); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	collection := parser.Session.DB(dbmodel.VoteMgoDb).C(dbmodel.VoteCollection(req.GardenId, p.ID))
	collection.EnsureIndexKey("uid")

	return reply, nil
}
