// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package vote

import (
	"context"
	"encoding/json"
	"fmt"
	"property-garden/errors"
	dbmodel "property-garden/model"
	"property-garden/parser"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkVoteListParam(req *pb_v1.VoteListRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}

	return nil

}

func VoteList(ctx context.Context, req *pb_v1.VoteListRequest) (reply *pb_v1.VoteListReply, err error) {

	reply = &pb_v1.VoteListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkVoteListParam(req)
	if err != nil {
		return nil, err
	}
	dbname := utils.GetGardenDbName(req.GardenId)
	p := dbmodel.NewVote(dbname)
	where := map[string]interface{}{
		//"garden_id":req.GardenId,
	}
	if req.Title != "" {
		where["title like"] = "%" + req.Title + "%"
	}
	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.VoteItem, len(list))
	for i, v := range list {

		item := &pb_v1.VoteItem{
			Title:     v.Title,
			Start:     v.Start,
			End:       v.End,
			Id:        v.ID,
			CreatedAt: v.CreatedAt.Unix(),
		}
		if v.Pics != "" {
			item.Pics = strings.Split(v.Pics, ";")
		}
		topics := []*pb_v1.VoteTopic{}
		json.Unmarshal([]byte(v.Topics), &topics)
		item.Topics = topics

		voteMgo := dbmodel.VoteAnswer{}
		exist, err := dbmodel.CheckDbExist(parser.Session, dbmodel.VoteMgoDb)
		if err != nil {
			return nil, errors.DataBaseError
		}
		if exist {
			collection := parser.Session.DB(dbmodel.VoteMgoDb).C(dbmodel.VoteCollection(req.GardenId, v.ID))
			count, err := voteMgo.Count(collection, nil)
			if err != nil {
				logger.Error("mgo",
					zap.String("error", err.Error()))
				return nil, errors.DataBaseError
			}
			item.UserCount = count
		}

		if len(item.Pics) == 0 {
			item.Pics = []string{parser.Conf.Oss.Protocol + "://" + parser.Conf.Oss.Endpoint + "/" + parser.Conf.Oss.FixBucket + "/" + parser.Conf.Oss.VoteObj}
		}

		reply.List[i] = item
	}

	return reply, nil
}
