// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package vote

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-garden/errors"
	"property-garden/impl/v1/oss_utils"
	dbmodel "property-garden/model"
	"property-garden/parser"
	pb_v1 "property-garden/pb/v1"
	"property-garden/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkVoteUpdateParam(req *pb_v1.VoteUpdateRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "id不能为空")
	case req.GardenId == 0:
		return status.Error(10003, "小区不能为空")
	case req.Title == "":
		return status.Error(10003, "标题不能为空")
	case req.Start < 1 || req.End < 1:
		return status.Error(10003, "开始和结束时间不能为空")
	case req.Start >= req.End:
		return status.Error(10003, "开始时间不能晚于结束时间")
	case req.End <= time.Now().Unix():
		return status.Error(10003, "结束时间不能早于当前时间")
	case len(req.Topics) == 0:
		return status.Error(10003, "题目不能为空")
	}
	pic := []string{}
	for _, v := range req.Pics {
		if oss_utils.IsFix(v) {
			continue
		}
		pic = append(pic, v)
	}
	req.Pics = pic

	flagM := map[string]bool{}
	for _, v := range req.Topics {
		if v.TopicType != TopicTypeMultiChoice && v.TopicType != TopicTypeSingleChoice {
			continue
		}
		for _, c := range v.ChoiceItems {
			if c.Flag == "" || c.Text == "" {
				return status.Error(10003, "选择题的选项和标签不能为空")
			}
			if flagM[c.Flag] {
				return status.Error(10003, "选择题的标签有重复")
			}
			flagM[c.Flag] = true
		}

	}
	return nil
}

func VoteUpdate(ctx context.Context, req *pb_v1.VoteUpdateRequest) (reply *pb_v1.VoteUpdateReply, err error) {

	reply = &pb_v1.VoteUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkVoteUpdateParam(req)
	if err != nil {
		return nil, err
	}

	dbname := utils.GetGardenDbName(req.GardenId)
	// 获取旧数据
	old := dbmodel.NewVote(dbname)
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	// todo 投票中不能修改

	collection := parser.Session.DB(dbmodel.VoteMgoDb).C(dbmodel.VoteCollection(req.GardenId, req.Id))
	count, err := collection.Count()
	if err != nil {
		return nil, errors.DataBaseError
	}
	if count > 0 {
		return nil, status.Error(10003, "已经开始投票，不能修改")
	}

	for i, _ := range req.Topics {
		req.Topics[i].Number = int64(i + 1)
	}
	// 更新新数据
	now := time.Now()
	vote := dbmodel.NewVote(dbname)
	topics, _ := json.Marshal(req.Topics)

	values := map[string]interface{}{
		"title":      req.Title,
		"start":      req.Start,
		"end":        req.End,
		"topics":     string(topics),
		"updated_at": now,
	}
	values["pics"] = ""
	if len(req.Pics) > 0 {
		values["pics"] = utils.StringJoin(req.Pics, ";")
	}

	where = map[string]interface{}{
		"id": req.Id,
	}
	db := database.DB().Begin()
	err = vote.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	if err := oss_utils.OssObjAdd(req.Pics, strings.Split(old.Pics, ";")); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()
	return reply, nil
}
