// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"time"
)

type TBuildingUnit struct {
	ID int64 `gorm:"column:id" json:"id"`
	BuildingNumber string `gorm:"column:building_number" json:"building_number"`
	BuildingName string `gorm:"column:building_name" json:"building_name"`
	BuildingArea float64 `gorm:"column:building_area" json:"building_area"`
	BuildingUsedArea float64 `gorm:"column:building_used_area" json:"building_used_area"`
	Comment string `gorm:"column:comment" json:"comment"`
	CreatedAt time.Time `gorm:"column:created_at" json:"created_at"`
	UpdatedAt time.Time `gorm:"column:updated_at" json:"updated_at"`
	UnitNumber int64 `gorm:"column:unit_number" json:"unit_number"`
	UnitName string `gorm:"column:unit_name" json:"unit_name"`
	UnitLayers int64 `gorm:"column:unit_layers" json:"unit_layers"`
	HasLift int64 `gorm:"column:has_lift" json:"has_lift"`
	UnitId int64 `gorm:"column:unit_id" json:"unit_id"`
	table string
}

func (p *TBuildingUnit) TableName() string {
	return p.table
}

func NewBuildingUnit(database string) *TBuildingUnit {
	return &TBuildingUnit{table: fmt.Sprintf("%s.%s", database, "t_building_unit")}
}

func (p *TBuildingUnit) CreateView(db *gorm.DB, database string) error {
	sql := "CREATE VIEW  "+database+".`t_building_unit` AS select t1.*,"+
		"`t2`.`unit_name` AS `unit_name`,`t2`.`has_lift` AS `has_lift`, t2.unit_number,"+
		"`t2`.`unit_layers` AS `unit_layers`,`t2`.`id` AS `unit_id` from "+
		"("+database+".`t_building` `t1` left join "+database+".`t_unit` `t2` on((`t1`.`id` = `t2`.`building_id`)))"
	err := db.Exec(sql).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "create view "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TBuildingUnit) Find(db *gorm.DB, where map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(where).Find(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return err
}

func (p *TBuildingUnit) Last(db *gorm.DB) error {
	err := db.Table(p.TableName()).Last(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select last from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}

	return err
}

// Insert 插入一条记录
func (p *TBuildingUnit) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}


func (p *TBuildingUnit) Delete(db *gorm.DB, filter map[string]interface{}) error {
	cond, val, err := whereBuild(filter)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *TBuildingUnit) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		if err != nil {
			fields, _ := util.MarshalToString(values)
			logger.Error("mysql",
				zap.String("sql", "update "+p.TableName()),
				zap.String("fields", fields),
				zap.String("error", err.Error()))
		}
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func (p *TBuildingUnit) UpdateByModel(db *gorm.DB) error {
	err := db.Model(p).Updates(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TBuildingUnit) BuildingCount(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Group("id").Count(&ret).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select count "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return ret, err
}

type BuildingItem struct {
	TBuildingUnit
	UnitCount int64 `gorm:"column:unit_count" json:"unit_count"`
}

func (p *TBuildingUnit) BuildingList(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int, pageSize int) (list []BuildingItem, err error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return list, err
	}
	if pageSize < 0 {
		result := db.Table(p.TableName()).Select([]string{"*, sum(unit_id > 0) as unit_count"}).Where(cond, val...).Group("id").Order("building_number").Find(&list)
		if result.Error != nil {
			wherefields, _ := util.MarshalToString(where)
			logger.Error("mysql",
				zap.String("sql", "select * from "+p.TableName()),
				zap.String("where", wherefields),
				zap.String("error", result.Error.Error()))
		}

		return list, result.Error
	}
	offset := (page - 1) * pageSize
	result := db.Table(p.TableName()).Select([]string{"*, sum(unit_id > 0) as unit_count"}).Where(cond, val...).Group("id").Order("building_number").Limit(pageSize).Offset(offset).Find(&list)
	if result.Error != nil {
		wherefields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select * from "+p.TableName()),
			zap.String("where", wherefields),
			zap.String("error", result.Error.Error()))
	}

	return list, result.Error

}

func (p *TBuildingUnit) UnitCount(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select count "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return ret, err
}


func (p *TBuildingUnit) UnitList(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int, pageSize int) (list []TBuildingUnit, err error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return list, err
	}
	if pageSize < 0 {
		result := db.Table(p.TableName()).Where(cond, val...).Order("building_number").Order("unit_number").Find(&list)
		if result.Error != nil {
			wherefields, _ := util.MarshalToString(where)
			logger.Error("mysql",
				zap.String("sql", "select * from "+p.TableName()),
				zap.String("where", wherefields),
				zap.String("error", result.Error.Error()))
		}

		return list, result.Error
	}
	offset := (page - 1) * pageSize
	result := db.Table(p.TableName()).Where(cond, val...).Order("building_number").Order("unit_number").Limit(pageSize).Offset(offset).Find(&list)
	if result.Error != nil {
		wherefields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select * from "+p.TableName()),
			zap.String("where", wherefields),
			zap.String("error", result.Error.Error()))
	}

	return list, result.Error

}
