// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"time"
)

type THouseholdUser struct {
	ID           int64     `gorm:"column:id" json:"id"`
	Phone        string    `gorm:"column:phone" json:"phone"`
	NickName     string    `gorm:"column:nick_name" json:"nick_name"`
	RealName     string    `gorm:"column:real_name" json:"real_name"`
	IdNumber     string    `gorm:"column:id_number" json:"id_number"`
	CreatedAt    time.Time `gorm:"column:created_at" json:"created_at"`
	UpdatedAt    time.Time `gorm:"column:updated_at" json:"updated_at"`
	OpenId       string    `gorm:"column:open_id" json:"open_id"`
	IdType       int32     `gorm:"column:id_type" json:"id_type"`
	UnionId      string    `gorm:"column:union_id" json:"union_id"`
	Avatar       string    `gorm:"column:avatar" json:"avatar"`
	PublicOpenId string    `gorm:"column:public_open_id" json:"public_open_id"`
	table        string
}

func (p *THouseholdUser) TableName() string {
	return p.table
}

func NewHouseholdUser(database string) *THouseholdUser {
	return &THouseholdUser{table: fmt.Sprintf("%s.%s", database, "t_household_user")}
}

func (p *THouseholdUser) SetTable(database string) {
	p.table = fmt.Sprintf("%s.%s", database, "t_household_user")
}

func (p *THouseholdUser) CreateTable(db *gorm.DB) error {
	sql := "CREATE TABLE IF NOT EXISTS " + p.TableName() + "(" +
		"`id` bigint(11) NOT NULL AUTO_INCREMENT," +
		"`phone` varchar(16) COLLATE utf8mb4_bin NOT NULL," +
		"`nick_name` varchar(255) COLLATE utf8mb4_bin NOT NULL," +
		"`real_name` varchar(64) COLLATE utf8mb4_bin NOT NULL," +
		"`id_number` varchar(64) COLLATE utf8mb4_bin NOT NULL," +
		"`created_at` datetime NOT NULL," +
		"`updated_at` datetime NOT NULL," +
		"`open_id` varchar(128) COLLATE utf8mb4_bin NOT NULL," +
		"`id_type` tinyint(1) NOT NULL COMMENT '1 身份证 2 港澳台居住证 3 港澳台同胞来往通行证 4 护照 5 军官证'," +
		"`union_id` varchar(128) COLLATE utf8mb4_bin NOT NULL," +
		"`avatar` varchar(255) COLLATE utf8mb4_bin NOT NULL COMMENT '微信头像'," +
		"`public_open_id` varchar(128) COLLATE utf8mb4_bin NOT NULL COMMENT '公众号openid'," +
		"PRIMARY KEY (`id`)," +
		"	UNIQUE KEY `phone` (`phone`,`open_id`) USING BTREE," +
		"UNIQUE KEY `open_id` (`open_id`) USING BTREE" +
		") ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin;"
	err := db.Exec(sql).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "create table "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *THouseholdUser) Find(db *gorm.DB, where map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(where).Find(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return err
}

func (p *THouseholdUser) Last(db *gorm.DB) error {
	err := db.Table(p.TableName()).Last(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select last from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}

	return err
}

// Insert 插入一条记录
func (p *THouseholdUser) Insert(db *gorm.DB) error {
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// Insert 插入多条记录
func (p *THouseholdUser) InsertMulti(db *gorm.DB, values interface{}) error {
	err := db.Table(p.TableName()).Create(values).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *THouseholdUser) Delete(db *gorm.DB, filter map[string]interface{}) error {
	cond, val, err := whereBuild(filter)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *THouseholdUser) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		if err != nil {
			fields, _ := util.MarshalToString(values)
			logger.Error("mysql",
				zap.String("sql", "update "+p.TableName()),
				zap.String("fields", fields),
				zap.String("error", err.Error()))
		}
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func (p *THouseholdUser) UpdateByModel(db *gorm.DB) error {
	err := db.Table(p.TableName()).Model(p).Updates(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *THouseholdUser) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select count "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return ret, err
}

func (p *THouseholdUser) List(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int, pageSize int) (list []THouseholdUser, err error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return list, err
	}
	if pageSize < 0 {
		result := db.Table(p.TableName()).Where(cond, val...).Find(&list)
		if result.Error != nil {
			wherefields, _ := util.MarshalToString(where)
			logger.Error("mysql",
				zap.String("sql", "select * from "+p.TableName()),
				zap.String("where", wherefields),
				zap.String("error", result.Error.Error()))
		}

		return list, result.Error
	}
	offset := (page - 1) * pageSize
	result := db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Find(&list)
	if result.Error != nil {
		wherefields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select * from "+p.TableName()),
			zap.String("where", wherefields),
			zap.String("error", result.Error.Error()))
	}

	return list, result.Error

}
