// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"time"
)

type TNeighborComment struct {
	ID              int64     `gorm:"column:id;PRIMARY_KEY" json:"id"`
	ArticleId       int64     `gorm:"column:article_id" json:"article_id"`
	ClassId         int64     `gorm:"column:class_id" json:"class_id"`
	Pid             int64     `gorm:"column:pid" json:"pid"`
	Content         string    `gorm:"column:content" json:"content"`
	Pics            string    `gorm:"column:pics" json:"pics"`
	Uid             int64     `gorm:"column:uid" json:"uid"`
	NickName        string    `gorm:"column:nick_name" json:"nick_name"`
	SubCommentCount int64     `gorm:"column:sub_comment_count" json:"sub_comment_count"`
	CreatedAt       time.Time `gorm:"column:created_at" json:"created_at"`
	UpdatedAt       time.Time `gorm:"column:updated_at" json:"updated_at"`
	Likes           int64     `gorm:"column:likes" json:"likes"`
	UserIcon        string    `gorm:"column:user_icon" json:"user_icon"`
	table           string
}

func (p *TNeighborComment) TableName() string {
	return p.table
}

func NewNeighborComment(database string) *TNeighborComment {
	return &TNeighborComment{table: fmt.Sprintf("%s.%s", database, "t_neighbor_comment")}
}

func (p *TNeighborComment) SetTable(database string) {
	p.table = fmt.Sprintf("%s.%s", database, "t_neighbor_comment")
}

func (p *TNeighborComment) CreateTable(db *gorm.DB) error {
	sql := "CREATE TABLE IF NOT EXISTS " + p.TableName() + "(" +
		"`id` bigint(11) NOT NULL AUTO_INCREMENT," +
		"`article_id` bigint(11) NOT NULL," +
		"`class_id` bigint(11) NOT NULL," +
		"`pid` bigint(11) NOT NULL," +
		"`content` text COLLATE utf8mb4_bin NOT NULL," +
		"`pics` text COLLATE utf8mb4_bin NOT NULL," +
		"`uid` bigint(20) NOT NULL," +
		"`sub_comment_count` int(10) NOT NULL," +
		"`created_at` datetime NOT NULL," +
		"`updated_at` datetime NOT NULL," +
		"`likes` int(10) NOT NULL," +
		"`user_icon` varchar(255) COLLATE utf8mb4_bin NOT NULL," +
		"  `nick_name` varchar(255) COLLATE utf8mb4_bin NOT NULL," +
		"PRIMARY KEY (`id`)," +
		"KEY `class_id` (`class_id`) USING BTREE," +
		"KEY `uid` (`uid`) USING BTREE," +
		"	KEY `article_id` (`article_id`) USING BTREE" +
		") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin;"
	err := db.Exec(sql).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "create table "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TNeighborComment) Find(db *gorm.DB, where map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(where).First(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return err
}

func (p *TNeighborComment) Last(db *gorm.DB) error {
	err := db.Table(p.TableName()).Last(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select last from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}

	return err
}

// Insert 插入一条记录
func (p *TNeighborComment) Insert(db *gorm.DB) error {
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// Insert 插入多条记录
func (p *TNeighborComment) InsertMulti(db *gorm.DB, values interface{}) error {
	err := db.Table(p.TableName()).Create(values).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TNeighborComment) Delete(db *gorm.DB, filter map[string]interface{}) error {
	cond, val, err := whereBuild(filter)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *TNeighborComment) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		if err != nil {
			fields, _ := util.MarshalToString(values)
			logger.Error("mysql",
				zap.String("sql", "update "+p.TableName()),
				zap.String("fields", fields),
				zap.String("error", err.Error()))
		}
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func (p *TNeighborComment) UpdateByModel(db *gorm.DB) error {
	err := db.Table(p.TableName()).Model(p).Updates(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TNeighborComment) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select count "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return ret, err
}

func (p *TNeighborComment) List(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int, pageSize int) (list []TNeighborComment, err error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return list, err
	}
	if pageSize < 0 {
		result := db.Table(p.TableName()).Where(cond, val...).Order("created_at desc").Find(&list)
		if result.Error != nil {
			wherefields, _ := util.MarshalToString(where)
			logger.Error("mysql",
				zap.String("sql", "select * from "+p.TableName()),
				zap.String("where", wherefields),
				zap.String("error", result.Error.Error()))
		}

		return list, result.Error
	}
	offset := (page - 1) * pageSize
	result := db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Order("created_at desc").Find(&list)
	if result.Error != nil {
		wherefields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select * from "+p.TableName()),
			zap.String("where", wherefields),
			zap.String("error", result.Error.Error()))
	}

	return list, result.Error

}
