// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"time"
)

type TParkSpace struct {
	ID int64 `gorm:"column:id;PRIMARY_KEY" json:"id"`
	SpaceNumber string `gorm:"column:space_number" json:"space_number"`
	SpaceType int32 `gorm:"column:space_type" json:"space_type"`
	SpaceArea float64 `gorm:"column:space_area" json:"space_area"`
	Comment string `gorm:"column:comment" json:"comment"`
	ParkId int64 `gorm:"column:park_id" json:"park_id"`
	Status int32 `gorm:"column:status" json:"status"`
	//GardenId int64 `gorm:"column:garden_id" json:"garden_id"`
	CreatedAt time.Time `gorm:"column:created_at" json:"created_at"`
	UpdatedAt time.Time `gorm:"column:updated_at" json:"updated_at"`
	BindStatus int64 `gorm:"column:bind_status" json:"bind_status"`
	VehicleId int64 `gorm:"column:vehicle_id" json:"vehicle_id"`
	HouseId int64 `gorm:"column:house_id" json:"house_id"`
	table string
}

func (p *TParkSpace) TableName() string {
	return p.table
}

func NewParkSpace(database string) *TParkSpace {
	return &TParkSpace{table: fmt.Sprintf("%s.%s", database, "t_park_space")}
}

func (p *TParkSpace) SetTable(database string)  {
	p.table = fmt.Sprintf("%s.%s", database, "t_park_space")
}

func (p *TParkSpace) CreateTable(db *gorm.DB) error {
	sql :="CREATE TABLE  IF NOT EXISTS "+p.TableName()+"("+
		"`id` bigint(11) NOT NULL AUTO_INCREMENT,"+
	"`space_number` varchar(32) COLLATE utf8mb4_bin NOT NULL,"+
	"`space_type` tinyint(1) NOT NULL COMMENT '1  标准车位 2 大型车位 3 小型车位 4 特殊车位',"+
	"`space_area` decimal(18,2) NOT NULL,"+
	"`comment` varchar(512) COLLATE utf8mb4_bin NOT NULL,"+
		"`status` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1 空闲 2 已售 3 已租',"+
	"`park_id` int(10) NOT NULL,"+
	"`created_at` datetime NOT NULL,"+
	"`updated_at` datetime NOT NULL,"+
	"	`bind_status` tinyint(1) NOT NULL COMMENT '1 空闲 2 已占用',"+
	"`vehicle_id` int(10) NOT NULL,"+
	"	`house_id` bigint(11) NOT NULL COMMENT '房屋id',"+
	"PRIMARY KEY (`id`) USING BTREE,"+
	"	UNIQUE KEY `park_space` (`park_id`,`space_number`) USING BTREE,"+
	"	KEY `vehicle_id` (`vehicle_id`) USING BTREE,"+
	"	KEY `house_id` (`house_id`) USING BTREE"+
	") ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=COMPACT COMMENT='停车位';"
	err := db.Exec(sql).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "create table "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return err
}


func (p *TParkSpace) Find(db *gorm.DB, where map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(where).Find(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return err
}

func (p *TParkSpace) Last(db *gorm.DB) error {
	err := db.Table(p.TableName()).Last(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select last from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}

	return err
}

// Insert 插入一条记录
func (p *TParkSpace) Insert(db *gorm.DB) error {
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TParkSpace) Delete(db *gorm.DB, filter map[string]interface{}) error {
	cond, val, err := whereBuild(filter)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *TParkSpace) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		if err != nil {
			fields, _ := util.MarshalToString(values)
			logger.Error("mysql",
				zap.String("sql", "update "+p.TableName()),
				zap.String("fields", fields),
				zap.String("error", err.Error()))
		}
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func (p *TParkSpace) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select count "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return ret, err
}

func (p *TParkSpace) List(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int, pageSize int) (list []TParkSpace, err error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return list, err
	}
	if pageSize < 0 {
		result := db.Table(p.TableName()).Where(cond, val...).Order("created_at desc").Find(&list)
		if result.Error != nil {
			wherefields, _ := util.MarshalToString(where)
			logger.Error("mysql",
				zap.String("sql", "select * from "+p.TableName()),
				zap.String("where", wherefields),
				zap.String("error", result.Error.Error()))
		}

		return list, result.Error
	}
	offset := (page - 1) * pageSize
	result := db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Order("created_at desc").Find(&list)
	if result.Error != nil {
		wherefields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select * from "+p.TableName()),
			zap.String("where", wherefields),
			zap.String("error", result.Error.Error()))
	}

	return list, result.Error

}


func (p *TParkSpace) GetIds(db *gorm.DB, where map[string]interface{}) ([]int64, map[int64]int64, error) {
	cond, val, err := whereBuildAndOr(where, nil)
	if err != nil {
		return nil, nil, err
	}
	type Result struct {
		Id int64
		ParkId int64
	}
	array := []Result{}
	err = db.Table(p.TableName()).Select("id").Where(cond, val...).Find(&array).Error
	if err != nil {
		wherefields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select id from "+p.TableName()),
			zap.String("where", wherefields),
			zap.String("error", err.Error()))
	}
	ret := make([]int64, len(array))
	m := map[int64]int64{}
	for i, v := range array {
		ret[i] = v.Id
		m[v.Id] = v.ParkId
	}
	return ret, m, err
}

