// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"time"
)

type TSuggestionOrder struct {
	ID                 int64     `gorm:"column:id" json:"id"`
	SuggestionType     int32     `gorm:"column:suggestion_type" json:"suggestion_type"`
	ApplyPeople        string    `gorm:"column:apply_people" json:"apply_people"`
	ApplyPeoplePhone   string    `gorm:"column:apply_people_phone" json:"apply_people_phone"`
	ApplyContent       string    `gorm:"column:apply_content" json:"apply_content"`
	ApplyPic           string    `gorm:"column:apply_pic" json:"apply_pic"`
	LastUid            int64     `gorm:"column:last_uid" json:"last_uid"`
	HandlePic          string    `gorm:"column:handle_pic" json:"handle_pic"`
	ReturnVisitContent string    `gorm:"column:return_visit_content" json:"return_visit_content"`
	ReturnVisitLevel   int32     `gorm:"column:return_visit_level" json:"return_visit_level"`
	Feedback           string    `gorm:"column:feedback" json:"feedback"`
	Status             int32     `gorm:"column:status" json:"status"`
	Pipeline           string    `gorm:"column:pipeline" json:"pipeline"`
	CurrentUid         int64     `gorm:"column:current_uid" json:"current_uid"`
	CreatedAt          time.Time `gorm:"column:created_at" json:"created_at"`
	UpdatedAt          time.Time `gorm:"column:updated_at" json:"updated_at"`
	LastUidIsCompany   bool     `gorm:"column:last_uid_is_company" json:"last_uid_is_company"`
	HouseholdUid       int64     `gorm:"column:household_uid" json:"household_uid"`
	table              string
}

func (p *TSuggestionOrder) TableName() string {
	return p.table
}

func NewSuggestionOrder(database string) *TSuggestionOrder {
	return &TSuggestionOrder{table: fmt.Sprintf("%s.%s", database, "t_suggestion_order")}
}

func (p *TSuggestionOrder) SetTable(database string) {
	p.table = fmt.Sprintf("%s.%s", database, "t_suggestion_order")
}

func (p *TSuggestionOrder) CreateTable(db *gorm.DB) error {
	sql := "CREATE TABLE IF NOT EXISTS " + p.TableName() + "(" +
		"`id` int(10) NOT NULL AUTO_INCREMENT," +
		"`suggestion_type` int(10) NOT NULL COMMENT '1 投诉 2 建议'," +
		"`apply_people` varchar(128) COLLATE utf8mb4_bin NOT NULL COMMENT '报修人'," +
		"`apply_people_phone` varchar(32) COLLATE utf8mb4_bin NOT NULL COMMENT '报修人电话'," +
		"`apply_content` text COLLATE utf8mb4_bin NOT NULL COMMENT '投诉内容'," +
		"`apply_pic` text COLLATE utf8mb4_bin NOT NULL COMMENT '投诉图片'," +
		"`last_uid` int(10) NOT NULL COMMENT '上级处理人uid'," +
		"`handle_pic` text COLLATE utf8mb4_bin NOT NULL COMMENT '处理图片'," +
		"`return_visit_content` text COLLATE utf8mb4_bin NOT NULL COMMENT '回访内容'," +
		"`return_visit_level` tinyint(1) NOT NULL COMMENT '回访满意度'," +
		"`feedback` text COLLATE utf8mb4_bin NOT NULL COMMENT '处理意见'," +
		"`status` tinyint(1) NOT NULL COMMENT '1 未派单 2 已派单 3 已完结'," +
		"`pipeline` text COLLATE utf8mb4_bin NOT NULL COMMENT '工单流水线详情'," +
		"`current_uid` int(10) NOT NULL COMMENT '当前处理人uid'," +
		"`created_at` datetime NOT NULL," +
		"	`updated_at` datetime NOT NULL," +
		"	`last_uid_is_company` tinyint(1) NOT NULL COMMENT '上级处理人是否是公司账户'," +
		"`household_uid` bigint(11) NOT NULL COMMENT '业主id，业主小程序添加任务时不为空'," +
		"PRIMARY KEY (`id`)," +
		"	KEY `apply_people` (`apply_people`) USING BTREE," +
		"	KEY `apply_people_phone` (`apply_people_phone`) USING BTREE," +
		"	KEY `household_uid` (`household_uid`) USING BTREE" +
		") ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin;"
	err := db.Exec(sql).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "create table "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TSuggestionOrder) Find(db *gorm.DB, where map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(where).Find(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return err
}

func (p *TSuggestionOrder) Last(db *gorm.DB) error {
	err := db.Table(p.TableName()).Last(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select last from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}

	return err
}

// Insert 插入一条记录
func (p *TSuggestionOrder) Insert(db *gorm.DB) error {
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// Insert 插入多条记录
func (p *TSuggestionOrder) InsertMulti(db *gorm.DB, values interface{}) error {
	err := db.Table(p.TableName()).Create(values).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TSuggestionOrder) Delete(db *gorm.DB, filter map[string]interface{}) error {
	cond, val, err := whereBuild(filter)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *TSuggestionOrder) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		if err != nil {
			fields, _ := util.MarshalToString(values)
			logger.Error("mysql",
				zap.String("sql", "update "+p.TableName()),
				zap.String("fields", fields),
				zap.String("error", err.Error()))
		}
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func (p *TSuggestionOrder) UpdateByModel(db *gorm.DB) error {
	err := db.Table(p.TableName()).Model(p).Updates(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TSuggestionOrder) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select count "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return ret, err
}

func (p *TSuggestionOrder) List(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int, pageSize int) (list []TSuggestionOrder, err error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return list, err
	}
	if pageSize < 0 {
		result := db.Table(p.TableName()).Where(cond, val...).Order("created_at desc").Find(&list)
		if result.Error != nil {
			wherefields, _ := util.MarshalToString(where)
			logger.Error("mysql",
				zap.String("sql", "select * from "+p.TableName()),
				zap.String("where", wherefields),
				zap.String("error", result.Error.Error()))
		}

		return list, result.Error
	}
	offset := (page - 1) * pageSize
	result := db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Order("created_at desc").Find(&list)
	if result.Error != nil {
		wherefields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select * from "+p.TableName()),
			zap.String("where", wherefields),
			zap.String("error", result.Error.Error()))
	}

	return list, result.Error

}
